@@ -12,12 +12,13 @@ mod_list = [mod for mod in sorted(os.listdir(root_dir)) if os.path.isdir(os.path
 
 pytest_base_cmd = '/opt/az/bin/python3 -m pytest -x -v --boxed -p no:warnings --log-level=WARN'
 pytest_parallel_cmd = '{} -n auto'.format(pytest_base_cmd)
+serial_test_modules = ['botservice', 'network', 'cloud', 'appservice']
 
 for mod_name in mod_list:
-    if mod_name in ['botservice', 'network', 'cloud']:
-        exit_code = subprocess.call(['{} --junit-xml /azure_cli_test_result/{}.xml --pyargs azure.cli.command_modules.{}'.format(pytest_base_cmd, mod_name, mod_name)], shell=True)
-    else:
-        exit_code = subprocess.call(['{} --junit-xml /azure_cli_test_result/{}.xml --pyargs azure.cli.command_modules.{}'.format(pytest_parallel_cmd, mod_name, mod_name)], shell=True)
+    cmd = '{} --junit-xml /azure_cli_test_result/{}.xml --pyargs azure.cli.command_modules.{}'.format(
+        pytest_base_cmd if mod_name in serial_test_modules else pytest_parallel_cmd, mod_name, mod_name)
+    print('Running:', cmd, flush=True)
+    exit_code = subprocess.call(cmd, shell=True)
     if exit_code == 5:
         print('No tests found for {}'.format(mod_name))
     elif exit_code != 0:
