@@ -18,12 +18,12 @@ CLASSIFIERS = [
     'Programming Language :: Python :: 3.6',
     'Programming Language :: Python :: 3.7',
     'Programming Language :: Python :: 3.8',
+    'Programming Language :: Python :: 3.9',
     'License :: OSI Approved :: MIT License',
 ]
 
 DEPENDENCIES = [
     'jmespath',
-    'mock',
     'vcrpy>=1.10.3',
     'azure-devtools==1.2.0',
     'pytest'
