@@ -25,6 +25,8 @@ ACR_NULL_CONTEXT = '/dev/null'
 
 ACR_TASK_QUICKTASK = 'quicktask'
 
+ACR_RUN_DEFAULT_TIMEOUT_IN_SEC = 60 * 60  # 60 minutes
+
 
 def get_classic_sku(cmd):
     SkuName = cmd.get_models('SkuName')
@@ -42,22 +44,22 @@ def get_premium_sku(cmd):
 
 
 def get_valid_os(cmd):
-    OS = cmd.get_models('OS')
+    OS = cmd.get_models('OS', operation_group='task_runs')
     return [item.value.lower() for item in OS]
 
 
 def get_valid_architecture(cmd):
-    Architecture = cmd.get_models('Architecture')
+    Architecture = cmd.get_models('Architecture', operation_group='task_runs')
     return [item.value.lower() for item in Architecture]
 
 
 def get_valid_variant(cmd):
-    Variant = cmd.get_models('Variant')
+    Variant = cmd.get_models('Variant', operation_group='task_runs')
     return [item.value.lower() for item in Variant]
 
 
 def get_finished_run_status(cmd):
-    RunStatus = cmd.get_models('RunStatus')
+    RunStatus = cmd.get_models('RunStatus', operation_group='task_runs')
     return [RunStatus.succeeded.value,
             RunStatus.failed.value,
             RunStatus.canceled.value,
@@ -66,22 +68,22 @@ def get_finished_run_status(cmd):
 
 
 def get_succeeded_run_status(cmd):
-    RunStatus = cmd.get_models('RunStatus')
+    RunStatus = cmd.get_models('RunStatus', operation_group='task_runs')
     return [RunStatus.succeeded.value]
 
 
 def get_acr_task_models(cmd):
     from azure.cli.core.profiles import get_sdk
-    return get_sdk(cmd.cli_ctx, ResourceType.MGMT_CONTAINERREGISTRY, 'models')
+    return get_sdk(cmd.cli_ctx, ResourceType.MGMT_CONTAINERREGISTRY, 'models', operation_group='tasks')
 
 
 def get_succeeded_agentpool_status(cmd):
-    AgentPoolStatus = cmd.get_models('ProvisioningState')
+    AgentPoolStatus = cmd.get_models('ProvisioningState', operation_group='agent_pools')
     return [AgentPoolStatus.succeeded.value]
 
 
 def get_finished_agentpool_status(cmd):
-    AgentPoolStatus = cmd.get_models('ProvisioningState')
+    AgentPoolStatus = cmd.get_models('ProvisioningState', operation_group='agent_pools')
     return [AgentPoolStatus.succeeded.value,
             AgentPoolStatus.failed.value,
             AgentPoolStatus.canceled.value]
