@@ -188,7 +188,7 @@ def load_command_table(self, _):  # pylint: disable=too-many-statements
         g.command('login', 'acr_login')
 
     with self.command_group('acr', acr_import_util) as g:
-        g.command('import', 'acr_import')
+        g.command('import', 'acr_import', supports_no_wait=True)
 
     with self.command_group('acr credential', acr_cred_util) as g:
         g.show_command('show', 'acr_credential_show')
@@ -359,9 +359,19 @@ def load_command_table(self, _):  # pylint: disable=too-many-statements
         g.show_command('show', 'acr_connected_registry_show')
         g.command('deactivate', 'acr_connected_registry_deactivate')
         g.command('update', 'acr_connected_registry_update')
-        g.command('install info', 'acr_connected_registry_install_info')
-        g.command('install renew-credentials', 'acr_connected_registry_install_renew_credentials')
+        g.command('get-settings', 'acr_connected_registry_get_settings')
+        g.command('permissions update', 'acr_connected_registry_permissions_update')
+        g.show_command('permissions show', 'acr_connected_registry_permissions_show',
+                       table_transformer=scope_map_output_format)
         g.command('list', 'acr_connected_registry_list',
                   table_transformer=connected_registry_list_output_format)
         g.command('list-client-tokens', 'acr_connected_registry_list_client_tokens',
                   table_transformer=token_output_format)
+        g.command('repo', 'acr_connected_registry_permissions_update',
+                  deprecate_info=self.deprecate(redirect='permissions update', hide=True))
+
+    with self.command_group('acr connected-registry install', acr_connected_registry_util,
+                            deprecate_info=self.deprecate(redirect='acr connected-registry get-settings',
+                                                          hide=True)) as g:
+        g.command('info', 'acr_connected_registry_install_info')
+        g.command('renew-credentials', 'acr_connected_registry_install_renew_credentials')
