@@ -245,6 +245,35 @@ class AcrCommandsTests(ScenarioTest):
         # test acr delete
         self.cmd('acr delete -n {registry_name} -g {rg} -y')
 
+    @ResourceGroupPreparer()
+    def test_acr_import_no_wait(self, resource_group):
+        source_registry_name = self.create_random_name("sourceregistrysamesub", 40)
+
+        self.kwargs.update({
+            'resource_id': '/subscriptions/dfb63c8c-7c89-4ef8-af13-75c1d873c895/resourcegroups/resourcegroupdiffsub/providers/Microsoft.ContainerRegistry/registries/sourceregistrydiffsub',
+            'source_registry_rg': 'resourcegroupsamesub',
+            'source_loc': 'westus',
+            'source_registry_name': source_registry_name,
+            'sku': 'Standard',
+            'source_image': 'microsoft:azure-cli',
+        })
+
+        # create a resource group for the source registry
+        self.cmd('group create -n {source_registry_rg} -l {source_loc}')
+
+        # create a source registry 
+        self.cmd('acr create -n {source_registry_name} -g {source_registry_rg} -l {source_loc} --sku {sku}',
+                 checks=[self.check('name', '{source_registry_name}'),
+                         self.check('location', '{source_loc}'),
+                         self.check('adminUserEnabled', False),
+                         self.check('sku.name', 'Standard'),
+                         self.check('sku.tier', 'Standard'),
+                         self.check('provisioningState', 'Succeeded')])
+
+        # import image using no-wait
+        self.cmd('acr import -n {source_registry_name} -r {resource_id} --source {source_image} --no-wait')
+
+    @AllowLargeResponse()
     @ResourceGroupPreparer()
     @record_only()
     def test_acr_image_import(self, resource_group):
@@ -263,8 +292,10 @@ class AcrCommandsTests(ScenarioTest):
         source_registry_name = self.create_random_name("sourceregistrysamesub", 40)
         registry_name = self.create_random_name("targetregistry", 20)
         token = self.cmd('account get-access-token').get_output_in_json()['accessToken']
-        service_principal_username = self.cmd('keyvault secret show --id https://imageimport.vault.azure.net/secrets/SPusername').get_output_in_json()['value']
-        service_principal_password = self.cmd('keyvault secret show --id https://imageimport.vault.azure.net/secrets/SPpassword').get_output_in_json()['value']
+
+        # service principal creds to support import from resource_imageV1
+        service_principal_username = self.cmd('keyvault secret show --id https://cliimportkv73021.vault.azure.net/secrets/SPusername').get_output_in_json()['value']
+        service_principal_password = self.cmd('keyvault secret show --id https://cliimportkv73021.vault.azure.net/secrets/SPpassword').get_output_in_json()['value']
 
         self.kwargs.update({
             'resource_id': '/subscriptions/dfb63c8c-7c89-4ef8-af13-75c1d873c895/resourcegroups/resourcegroupdiffsub/providers/Microsoft.ContainerRegistry/registries/sourceregistrydiffsub',
@@ -338,6 +369,55 @@ class AcrCommandsTests(ScenarioTest):
         # Case 9: Import image from an Azure Container Registry with personal access token
         self.cmd('acr import -n {registry_name} --source {resource_imageV2} -p {token}')
 
+
+    @ResourceGroupPreparer()
+    def test_acr_export_policy(self, resource_group):
+        registry_1 = self.create_random_name('clireg', 20)
+        registry_2 = self.create_random_name('clireg', 20)
+
+        self.kwargs.update({
+            'registry_1': registry_1,
+            'registry_2': registry_2,
+            'sku': 'Premium',
+        })
+
+        self.cmd('acr create -n {registry_1} -g {rg} --sku {sku} --public-network-enabled false --allow-exports',
+                 checks=[self.check('name', '{registry_1}'),
+                         self.check('provisioningState', 'Succeeded'),
+                         self.check_pattern('policies.exportPolicy.status', 'Enabled')])  # case insensitive check
+
+        self.cmd('acr create -n {registry_1} -g {rg} --sku {sku} --public-network-enabled false --allow-exports true',
+                 checks=[self.check('name', '{registry_1}'),
+                         self.check('provisioningState', 'Succeeded'),
+                         self.check_pattern('policies.exportPolicy.status', 'enabled')])
+
+        # for export policy to be disabled, we need to disable public network access
+        self.cmd('acr create -n {registry_1} -g {rg} --sku {sku} --public-network-enabled false --allow-exports false',
+                 checks=[self.check('name', '{registry_1}'),
+                         self.check('provisioningState', 'Succeeded'),
+                         self.check_pattern('policies.exportPolicy.status', 'Disabled')])
+
+        # create/PUT should default to enabling export policy
+        self.cmd('acr create -n {registry_1} -g {rg} --sku {sku} --public-network-enabled false',
+                 checks=[self.check('name', '{registry_1}'),
+                         self.check('provisioningState', 'Succeeded'),
+                         self.check_pattern('policies.exportPolicy.status', 'Enabled')])
+
+        self.cmd('acr update -n {registry_1} -g {rg} --sku {sku} --allow-exports true',
+                 checks=[self.check('name', '{registry_1}'),
+                         self.check('provisioningState', 'Succeeded'),
+                         self.check_pattern('policies.exportPolicy.status', 'enabled')])
+
+        self.cmd('acr update -n {registry_1} -g {rg} --sku {sku} --allow-exports false',
+                 checks=[self.check('name', '{registry_1}'),
+                         self.check('provisioningState', 'Succeeded'),
+                         self.check_pattern('policies.exportPolicy.status', 'disabled')])
+
+        self.cmd('acr update -n {registry_1} -g {rg} --sku {sku}',
+                 checks=[self.check('name', '{registry_1}'),
+                         self.check('provisioningState', 'Succeeded'),
+                         self.check_pattern('policies.exportPolicy.status', 'Disabled')])
+
     @ResourceGroupPreparer()
     def test_acr_create_with_audits(self, resource_group):
         registry_name = self.create_random_name('clireg', 20)
@@ -386,7 +466,7 @@ class AcrCommandsTests(ScenarioTest):
 
         # create the registry with CMK encryption enabled using the user-assigned identity
         result = self.cmd('acr create --name {registry_name} --resource-group {rg} --sku premium --identity {identity_id} --key-encryption-key {key_id}', checks=[
-            self.check('identity.type', 'UserAssigned'),
+            self.check('identity.type', 'userAssigned'),
             self.check('encryption.status', 'enabled'),
             self.check('encryption.keyVaultProperties.identity', '{client_id}'),
             self.check('encryption.keyVaultProperties.keyIdentifier', '{key_id}')
@@ -427,8 +507,8 @@ class AcrCommandsTests(ScenarioTest):
 
         # show identity
         result = self.cmd('acr identity show --name {registry_name}').get_output_in_json()
-        self.assertTrue('SystemAssigned' in result['type'])
-        self.assertTrue('UserAssigned' in result['type'])
+        self.assertTrue('systemAssigned' in result['type'])
+        self.assertTrue('userAssigned' in result['type'])
         self.assertTrue(len(result['userAssignedIdentities']) == 1)
         self.assertEquals(list(result['userAssignedIdentities'].keys())[0].lower(), self.kwargs['identity_id'].lower())
 
@@ -439,29 +519,29 @@ class AcrCommandsTests(ScenarioTest):
 
         # try different combinations of adds and deletes
         # system
-        self.cmd('acr identity assign --name {registry_name} --identities {system_identity}', self.check('identity.type', 'SystemAssigned'))
+        self.cmd('acr identity assign --name {registry_name} --identities {system_identity}', self.check('identity.type', 'systemAssigned'))
         time.sleep(10)
         self.cmd('acr identity remove --name {registry_name} --identities {system_identity}', self.check('identity', None))
         # user
-        self.cmd('acr identity assign --name {registry_name} --identities {identity_id}', self.check('identity.type', 'UserAssigned'))
+        self.cmd('acr identity assign --name {registry_name} --identities {identity_id}', self.check('identity.type', 'userAssigned'))
         time.sleep(10)
         self.cmd('acr identity remove --name {registry_name} --identities {identity_id}', self.check('identity', None))
 
         # add multiple identities
         result = self.cmd('acr identity assign --name {registry_name} --identities "{system_identity}" "{identity_id}"',
-                          self.check('identity.type', 'SystemAssigned, UserAssigned')).get_output_in_json()
+                          self.check('identity.type', 'systemAssigned, userAssigned')).get_output_in_json()
         self.assertUserIdentitiesExpected([self.kwargs['identity_id'].lower()], result['identity'])
         # add another user identity to existing
         time.sleep(10)
         result = self.cmd('acr identity assign --name {registry_name} --identities {second_identity_id}',
-                          self.check('identity.type', 'SystemAssigned, UserAssigned')).get_output_in_json()
+                          self.check('identity.type', 'systemAssigned, userAssigned')).get_output_in_json()
         self.assertUserIdentitiesExpected([self.kwargs['identity_id'].lower(), self.kwargs['second_identity_id'].lower()], result['identity'])
 
         # remove identities and validate result
         time.sleep(10)
-        self.cmd('acr identity remove --name {registry_name} --identities {second_identity_id}', self.check('identity.type', 'SystemAssigned, UserAssigned'))
+        self.cmd('acr identity remove --name {registry_name} --identities {second_identity_id}', self.check('identity.type', 'systemAssigned, userAssigned'))
 
-        self.cmd('acr identity remove --name {registry_name} --identities {identity_id}', self.check('identity.type', 'SystemAssigned'))
+        self.cmd('acr identity remove --name {registry_name} --identities {identity_id}', self.check('identity.type', 'systemAssigned'))
         self.cmd('acr identity remove --name {registry_name} --identities {system_identity}', self.check('identity', None))
 
     @ResourceGroupPreparer()
@@ -520,7 +600,19 @@ class AcrCommandsTests(ScenarioTest):
                  checks=[self.check('publicNetworkAccess', 'Disabled'),
                          self.check('networkRuleBypassOptions', 'None')])
 
-    @ResourceGroupPreparer(location='centraluseuap')
+    @ResourceGroupPreparer()
+    def test_acr_with_anonymous_pull(self, resource_group, resource_group_location):
+        self.kwargs.update({
+            'registry_name': self.create_random_name('testreg', 20)
+        })
+        self.cmd('acr create --name {registry_name} --resource-group {rg} --sku premium -l eastus',
+                 checks=[self.check('anonymousPullEnabled', False)])
+        self.cmd('acr update --name {registry_name} --resource-group {rg} --anonymous-pull-enabled true',
+                 checks=[self.check('anonymousPullEnabled', True)])
+        self.cmd('acr update --name {registry_name} --resource-group {rg} --anonymous-pull-enabled false',
+                 checks=[self.check('anonymousPullEnabled', False)])
+
+    @ResourceGroupPreparer(location='eastus2')
     def test_acr_with_private_endpoint(self, resource_group):
         self.kwargs.update({
             'registry_name': self.create_random_name('testreg', 20),
