@@ -3,6 +3,8 @@
 # Licensed under the MIT License. See License.txt in the project root for license information.
 # --------------------------------------------------------------------------------------------
 
+from enum import Enum
+
 CONST_OUTBOUND_TYPE_LOAD_BALANCER = "loadBalancer"
 CONST_OUTBOUND_TYPE_USER_DEFINED_ROUTING = "userDefinedRouting"
 
@@ -31,13 +33,45 @@ CONST_INGRESS_APPGW_SUBNET_ID = "subnetId"
 CONST_INGRESS_APPGW_SUBNET_CIDR = "subnetCIDR"
 CONST_INGRESS_APPGW_WATCH_NAMESPACE = "watchNamespace"
 
+# confcom addon keys
+CONST_CONFCOM_ADDON_NAME = "ACCSGXDevicePlugin"
+CONST_ACC_SGX_QUOTE_HELPER_ENABLED = "ACCSGXQuoteHelperEnabled"
+
+# Azure Keyvault Secrets Provider configuration keys
+CONST_AZURE_KEYVAULT_SECRETS_PROVIDER_ADDON_NAME = "azureKeyvaultSecretsProvider"
+CONST_SECRET_ROTATION_ENABLED = "enableSecretRotation"
+CONST_ROTATION_POLL_INTERVAL = "rotationPollInterval"
+
+# private dns zone mode
+CONST_PRIVATE_DNS_ZONE_SYSTEM = "system"
+CONST_PRIVATE_DNS_ZONE_NONE = "none"
+
+# Open Service Mesh addon keys
+CONST_OPEN_SERVICE_MESH_ADDON_NAME = "openServiceMesh"
+
 ADDONS = {
     'http_application_routing': CONST_HTTP_APPLICATION_ROUTING_ADDON_NAME,
     'monitoring': CONST_MONITORING_ADDON_NAME,
     'virtual-node': CONST_VIRTUAL_NODE_ADDON_NAME,
     'kube-dashboard': CONST_KUBE_DASHBOARD_ADDON_NAME,
     'azure-policy': CONST_AZURE_POLICY_ADDON_NAME,
-    'ingress-appgw': CONST_INGRESS_APPGW_ADDON_NAME
+    'ingress-appgw': CONST_INGRESS_APPGW_ADDON_NAME,
+    "confcom": CONST_CONFCOM_ADDON_NAME,
+    'open-service-mesh': CONST_OPEN_SERVICE_MESH_ADDON_NAME,
+    'azure-keyvault-secrets-provider': CONST_AZURE_KEYVAULT_SECRETS_PROVIDER_ADDON_NAME
 }
 
-CONST_CANIPULL_IMAGE = "mcr.microsoft.com/aks/canipull:0.0.2-alpha"
+CONST_CANIPULL_IMAGE = "mcr.microsoft.com/aks/canipull:0.0.3-alpha"
+
+CONST_MANAGED_IDENTITY_OPERATOR_ROLE = 'Managed Identity Operator'
+CONST_MANAGED_IDENTITY_OPERATOR_ROLE_ID = 'f1a07417-d97a-45cb-824c-7a7467783830'
+
+
+# decorator mode
+class DecoratorMode(Enum):
+    CREATE = 1
+    UPDATE = 2
+
+
+class DecoratorEarlyExitException(Exception):
+    pass
