@@ -13,12 +13,13 @@ from ipaddress import ip_network
 # pylint: disable=no-name-in-module,import-error
 from knack.log import get_logger
 
+from azure.cli.core.profiles import ResourceType
+
 from azure.cli.core.commands.validators import validate_tag
 from azure.cli.core.util import CLIError
 from azure.cli.core.azclierror import InvalidArgumentValueError
 import azure.cli.core.keys as keys
 
-from azure.mgmt.containerservice.v2020_09_01.models import ManagedClusterPropertiesAutoScalerProfile
 
 logger = get_logger(__name__)
 
@@ -107,7 +108,7 @@ def validate_k8s_version(namespace):
                            'such as "1.11.8" or "1.12.6"')
 
 
-def validate_cluster_autoscaler_profile(namespace):
+def validate_cluster_autoscaler_profile(cmd, namespace):
     """ Validates that cluster autoscaler profile is acceptable by:
         1. Extracting the key[=value] format to map
         2. Validating that the key isn't empty and that the key is valid
@@ -116,12 +117,15 @@ def validate_cluster_autoscaler_profile(namespace):
     _extract_cluster_autoscaler_params(namespace)
     if namespace.cluster_autoscaler_profile is not None:
         for key in namespace.cluster_autoscaler_profile.keys():
-            _validate_cluster_autoscaler_key(key)
+            _validate_cluster_autoscaler_key(cmd, key)
 
 
-def _validate_cluster_autoscaler_key(key):
+def _validate_cluster_autoscaler_key(cmd, key):
     if not key:
         raise CLIError('Empty key specified for cluster-autoscaler-profile')
+    ManagedClusterPropertiesAutoScalerProfile = cmd.get_models('ManagedClusterPropertiesAutoScalerProfile',
+                                                               resource_type=ResourceType.MGMT_CONTAINERSERVICE,
+                                                               operation_group='managed_clusters')
     valid_keys = list(k.replace("_", "-") for k, v in ManagedClusterPropertiesAutoScalerProfile._attribute_map.items())  # pylint: disable=protected-access
     if key not in valid_keys:
         raise CLIError("'{0}' is an invalid key for cluster-autoscaler-profile. "
@@ -417,3 +421,67 @@ def validate_assign_identity(namespace):
         from msrestazure.tools import is_valid_resource_id
         if not is_valid_resource_id(namespace.assign_identity):
             raise InvalidArgumentValueError("--assign-identity is not a valid Azure resource ID.")
+
+
+def validate_assign_kubelet_identity(namespace):
+    if namespace.assign_kubelet_identity is not None:
+        if namespace.assign_kubelet_identity == '':
+            return
+        from msrestazure.tools import is_valid_resource_id
+        if not is_valid_resource_id(namespace.assign_kubelet_identity):
+            raise InvalidArgumentValueError("--assign-kubelet-identity is not a valid Azure resource ID.")
+
+
+def extract_comma_separated_string(
+    raw_string,
+    enable_strip=False,
+    extract_kv=False,
+    key_only=False,
+    keep_none=False,
+    default_value=None,
+):
+    """Extract comma-separated string.
+
+    If enable_strip is specified, will remove leading and trailing whitespace before each operation on the string.
+    If extract_kv is specified, will extract key value pair from the string, otherwise keep the entire string.
+    Option key_only is valid since extract_kv is specified, when the number of string segments split by "=" is not 2,
+    only the first segment is retained as the key.
+    If keep_none is specified, will return None when input is None. Otherwise will return default_value if input is
+    None or empty string.
+    """
+    if raw_string is None:
+        if keep_none:
+            return None
+        return default_value
+    if enable_strip:
+        raw_string = raw_string.strip()
+    if raw_string == "":
+        return default_value
+
+    result = {} if extract_kv else []
+    for item in raw_string.split(","):
+        if enable_strip:
+            item = item.strip()
+        if extract_kv:
+            kv = item.split("=")
+            if key_only:
+                if enable_strip:
+                    result[kv[0].strip()] = ""
+                else:
+                    result[kv[0]] = ""
+            else:
+                if len(kv) == 2:
+                    if enable_strip:
+                        result[kv[0].strip()] = kv[1].strip()
+                    else:
+                        result[kv[0]] = kv[1]
+                else:
+                    raise InvalidArgumentValueError(
+                        "The format of '{}' in '{}' is incorrect, correct format should be "
+                        "'Key1=Value1,Key2=Value2'.".format(
+                            item, raw_string
+                        )
+                    )
+        else:
+            result.append(item)
+    return result
