@@ -3,9 +3,45 @@
 # Licensed under the MIT License. See License.txt in the project root for license information.
 # --------------------------------------------------------------------------------------------
 import unittest
+from unittest import mock
+from azure.cli.core.azclierror import CLIInternalError, InvalidArgumentValueError
+
+from knack import CLI
+
+from azure.cli.core._config import GLOBAL_CONFIG_DIR, ENV_VAR_PREFIX
+from azure.cli.core.cloud import get_active_cloud
+from azure.cli.core.profiles import get_sdk, ResourceType, supported_api_version
+
 from azure.cli.core.util import CLIError
 from azure.cli.command_modules.acs import _validators as validators
 
+class MockCLI(CLI):
+    def __init__(self):
+        super(MockCLI, self).__init__(cli_name='mock_cli', config_dir=GLOBAL_CONFIG_DIR,
+                                      config_env_var_prefix=ENV_VAR_PREFIX, commands_loader_cls=MockLoader)
+        self.cloud = get_active_cloud(self)
+
+
+class MockLoader(object):
+    def __init__(self, ctx):
+        self.ctx = ctx
+
+    def get_models(self, *attr_args, **_):
+        from azure.cli.core.profiles import get_sdk
+        return get_sdk(self.ctx, ResourceType.MGMT_CONTAINERSERVICE, 'ManagedClusterPropertiesAutoScalerProfile',
+                       mod='models', operation_group='managed_clusters')
+
+
+class MockCmd(object):
+    def __init__(self, ctx, arguments={}):
+        self.cli_ctx = ctx
+        self.loader = MockLoader(self.cli_ctx)
+        self.arguments = arguments
+
+    def get_models(self, *attr_args, **kwargs):
+        return get_sdk(self.cli_ctx, ResourceType.MGMT_CONTAINERSERVICE, 'ManagedClusterPropertiesAutoScalerProfile',
+                       mod='models', operation_group='managed_clusters')
+
 
 class TestValidateIPRanges(unittest.TestCase):
     def test_simultaneous_allow_and_disallow_with_spaces(self):
@@ -62,20 +98,23 @@ class TestValidateIPRanges(unittest.TestCase):
 
 
 class TestClusterAutoscalerParamsValidators(unittest.TestCase):
+    def setUp(self):
+        self.cli = MockCLI()
+
     def test_empty_key_empty_value(self):
         cluster_autoscaler_profile = ["="]
         namespace = Namespace(cluster_autoscaler_profile=cluster_autoscaler_profile)
         err = "Empty key specified for cluster-autoscaler-profile"
 
         with self.assertRaises(CLIError) as cm:
-            validators.validate_cluster_autoscaler_profile(namespace)
+            validators.validate_cluster_autoscaler_profile(MockCmd(self.cli), namespace)
         self.assertEqual(str(cm.exception), err)
 
     def test_non_empty_key_empty_value(self):
         cluster_autoscaler_profile = ["scan-interval="]
         namespace = Namespace(cluster_autoscaler_profile=cluster_autoscaler_profile)
 
-        validators.validate_cluster_autoscaler_profile(namespace)
+        validators.validate_cluster_autoscaler_profile(MockCmd(self.cli), namespace)
 
     def test_two_empty_keys_empty_value(self):
         cluster_autoscaler_profile = ["=", "="]
@@ -83,7 +122,7 @@ class TestClusterAutoscalerParamsValidators(unittest.TestCase):
         err = "Empty key specified for cluster-autoscaler-profile"
 
         with self.assertRaises(CLIError) as cm:
-            validators.validate_cluster_autoscaler_profile(namespace)
+            validators.validate_cluster_autoscaler_profile(MockCmd(self.cli), namespace)
         self.assertEqual(str(cm.exception), err)
 
     def test_one_empty_key_in_pair_one_non_empty(self):
@@ -92,7 +131,7 @@ class TestClusterAutoscalerParamsValidators(unittest.TestCase):
         err = "Empty key specified for cluster-autoscaler-profile"
 
         with self.assertRaises(CLIError) as cm:
-            validators.validate_cluster_autoscaler_profile(namespace)
+            validators.validate_cluster_autoscaler_profile(MockCmd(self.cli), namespace)
         self.assertEqual(str(cm.exception), err)
 
     def test_invalid_key(self):
@@ -101,14 +140,14 @@ class TestClusterAutoscalerParamsValidators(unittest.TestCase):
         err = "'bad-key' is an invalid key for cluster-autoscaler-profile"
 
         with self.assertRaises(CLIError) as cm:
-            validators.validate_cluster_autoscaler_profile(namespace)
+            validators.validate_cluster_autoscaler_profile(MockCmd(self.cli), namespace)
         self.assertIn(err, str(cm.exception),)
 
     def test_valid_parameters(self):
         cluster_autoscaler_profile = ["scan-interval=20s", "scale-down-delay-after-add=15m"]
         namespace = Namespace(cluster_autoscaler_profile=cluster_autoscaler_profile)
 
-        validators.validate_cluster_autoscaler_profile(namespace)
+        validators.validate_cluster_autoscaler_profile(MockCmd(self.cli), namespace)
 
 
 class Namespace:
@@ -303,3 +342,65 @@ class TestAssignIdentity(unittest.TestCase):
         empty_identity_id = ""
         namespace = AssignIdentityNamespace(empty_identity_id)
         validators.validate_assign_identity(namespace)
+
+
+class TestExtractCommaSeparatedString(unittest.TestCase):
+    def test_extract_comma_separated_string(self):
+        s1 = "abc, xyz, 123"
+        t1 = validators.extract_comma_separated_string(s1)
+        g1 = ["abc", " xyz", " 123"]
+        self.assertEqual(t1, g1)
+
+        s2 = "abc, xyz, 123"
+        t2 = validators.extract_comma_separated_string(s2, enable_strip=True)
+        g2 = ["abc", "xyz", "123"]
+        self.assertEqual(t2, g2)
+
+        s3 = None
+        t3 = validators.extract_comma_separated_string(s3, keep_none=True, default_value="")
+        g3 = None
+        self.assertEqual(t3, g3)
+
+        s4 = None
+        t4 = validators.extract_comma_separated_string(s4, keep_none=False, default_value="")
+        g4 = ""
+        self.assertEqual(t4, g4)
+
+        s5 = ""
+        t5 = validators.extract_comma_separated_string(s5, keep_none=True, default_value={})
+        g5 = {}
+        self.assertEqual(t5, g5)
+
+        s6 = "abc = def, xyz = 123"
+        t6 = validators.extract_comma_separated_string(s6, extract_kv=True)
+        g6 = {"abc ": " def", " xyz ": " 123"}
+        self.assertEqual(t6, g6)
+
+        s7 = "abc = def, xyz = 123"
+        t7 = validators.extract_comma_separated_string(s7, enable_strip=True, extract_kv=True)
+        g7 = {"abc": "def", "xyz": "123"}
+        self.assertEqual(t7, g7)
+
+        s8 = "abc = def, xyz = 123"
+        t8 = validators.extract_comma_separated_string(s8, extract_kv=True, key_only=True)
+        g8 = {"abc ": "", " xyz ": ""}
+        self.assertEqual(t8, g8)
+
+        s9 = "abc = def, xyz = 123"
+        t9 = validators.extract_comma_separated_string(s9, enable_strip=True, extract_kv=True, key_only=True)
+        g9 = {"abc": "", "xyz": ""}
+        self.assertEqual(t9, g9)
+
+        s10 = "abc def, xyz 123"
+        t10 = validators.extract_comma_separated_string(s10, enable_strip=True, extract_kv=True, key_only=True)
+        g10 = {"abc def": "", "xyz 123": ""}
+        self.assertEqual(t10, g10)
+
+        s11 = "abc def, xyz 123"
+        with self.assertRaises(InvalidArgumentValueError):
+            validators.extract_comma_separated_string(s11, enable_strip=True, extract_kv=True)
+
+        s12 = "WindowsContainerRuntime=containerd,AKSHTTPCustomFeatures=Microsoft.ContainerService/CustomNodeConfigPreview"
+        t12 = validators.extract_comma_separated_string(s12, enable_strip=True, extract_kv=True, default_value={},)
+        g12 = {"WindowsContainerRuntime": "containerd", "AKSHTTPCustomFeatures": "Microsoft.ContainerService/CustomNodeConfigPreview"}
+        self.assertEqual(t12, g12)
