@@ -64,13 +64,13 @@ def load_arguments(self, _):
         c.argument('backup_name', help='The name of the backup file to create.')
         c.argument('storage_account_name', arg_group='Storage', help='The name of the storage account used to place the backup.')
         c.argument('storage_account_key', arg_group='Storage', help='The access key of the storage account used to place the backup.')
-        c.argument('storage_account_container', options_list=['--storage-account-container', '--container-name'], nargs='+', help='The name of the storage account container used to place the backup.')
+        c.argument('storage_account_container', options_list=['--storage-account-container', '--container-name'], help='The name of the storage account container used to place the backup.')
 
     with self.argument_context('apim restore') as c:
         c.argument('backup_name', help='The name of the backup file to restore.')
         c.argument('storage_account_name', arg_group='Storage', help='The name of the storage account used to retrieve the backup from.')
         c.argument('storage_account_key', arg_group='Storage', help='The access key of the storage account used to retrieve the backup from.')
-        c.argument('storage_account_container', options_list=['--storage-account-container', '--container-name'], nargs='+', help='The name of the storage account container used to retrieve the backup from.')
+        c.argument('storage_account_container', options_list=['--storage-account-container', '--container-name'], help='The name of the storage account container used to retrieve the backup from.')
 
     with self.argument_context('apim api show') as c:
         c.argument('service_name', options_list=['--service-name'], help='The name of the API Management service instance.')
@@ -124,19 +124,21 @@ def load_arguments(self, _):
     with self.argument_context('apim api import') as c:
         c.argument('service_name', options_list=['--service-name', '-n'], help="The name of the api management service instance", id_part=None)
         c.argument('path', arg_group='API', help='Required. Relative URL uniquely identifying this API and all of its resource paths within the API Management service instance.', required=True)
+        c.argument('specification_format', arg_type=get_enum_type(ImportFormat), help='Specify the format of the imported API.')
         c.argument('description', arg_group='API', help='Description of the API. May include HTML formatting tags.')
         c.argument('subscription_key_header_name', arg_group='API', help='Specifies the subscription key header name.')
         c.argument('subscription_key_query_param_name', arg_group='API', help='Specifies the subscription key query string parameter name.')
         c.argument('api_id', arg_group='API', help='API revision identifier. Must be unique in the current API Management service instance. Non-current revision has ;rev=n as a suffix where n is the revision number.')
         c.argument('api_revision', arg_group='API', help='Describes the Revision of the Api. If no value is provided, default revision 1 is created.')
+        c.argument('api_version', arg_group='API', help='Describes the Version of the Api. If you add a version to a non-versioned API, an Original version will be automatically created and will respond on the default URL')
+        c.argument('api_version_set_id', arg_group='API', help='Describes the Version Set to be used with the API')
         c.argument('service_url', arg_group='API', help='Absolute URL of the backend service implementing this API. Cannot be more than 2000 characters long.')
-        c.argument('protocols', arg_group='API', arg_type=get_enum_type(API_PROTOCOLS), nargs='+', help='Describes on which protocols the operations in this API can be invoked.')
+        c.argument('protocols', arg_group='API', arg_type=get_enum_type(API_PROTOCOLS), nargs='+', help='Describes on which protocols(one or more) the operations in this API can be invoked.')
         c.argument('api_type', arg_group='API', arg_type=get_enum_type(API_TYPES), help='The type of the API.')
         c.argument('subscription_required', arg_group='API', arg_type=get_three_state_flag(), help='If true, the API requires a subscription key on requests.')
         c.argument('display_name', arg_group='API', help='Display name of this API.')
         c.argument('specification_path', help='File path specified to import the API.')
         c.argument('specification_url', help='Url specified to import the API.')
-        c.argument('specification_format', arg_type=get_enum_type(ImportFormat), help='Specify the format of the imported API.')
         c.argument('soap_api_type', help='The type of API when file format is WSDL.')
         c.argument('wsdl_service_name', help='Local name of WSDL Service to be imported.')
         c.argument('wsdl_endpoint_name', help='Local name of WSDL Endpoint (port) to be imported.')
