@@ -3,11 +3,13 @@
 # Licensed under the MIT License. See License.txt in the project root for license information.
 # --------------------------------------------------------------------------------------------
 
-# pylint: disable=line-too-long,too-many-nested-blocks
+# pylint: disable=line-too-long,too-many-nested-blocks,too-many-lines
 
 import io
 import json
-import re
+from difflib import Differ
+from json import JSONDecodeError
+from urllib.parse import urlparse
 
 import chardet
 import javaproperties
@@ -15,11 +17,15 @@ import yaml
 from jsondiff import JsonDiffer
 from knack.log import get_logger
 from knack.util import CLIError
-from azure.appconfiguration import ResourceReadOnlyError
-from azure.core.exceptions import HttpResponseError
 
-from ._constants import (FeatureFlagConstants, KeyVaultConstants)
-from ._utils import user_confirmation, prep_label_filter_for_url_encoding
+from azure.keyvault.key_vault_id import KeyVaultIdentifier
+from azure.appconfiguration import ResourceReadOnlyError, ConfigurationSetting
+from azure.core.exceptions import HttpResponseError
+from azure.cli.core.util import user_confirmation
+from azure.cli.core.azclierror import FileOperationError, AzureInternalError
+
+from ._constants import (FeatureFlagConstants, KeyVaultConstants, SearchFilterOptions, KVSetConstants, ImportExportProfiles)
+from ._utils import prep_label_filter_for_url_encoding
 from ._models import (KeyValue, convert_configurationsetting_to_keyvalue,
                       convert_keyvalue_to_configurationsetting, QueryFields)
 from._featuremodels import (map_keyvalue_to_featureflag,
@@ -106,10 +112,8 @@ def validate_import_key(key):
 
 def validate_import_feature(feature):
     if feature:
-        invalid_pattern = re.compile(r'[^a-zA-Z0-9._-]')
-        invalid = re.search(invalid_pattern, feature)
-        if invalid:
-            logger.warning("Ignoring invalid feature '%s'. Only alphanumeric characters, '.', '-' and '_' are allowed in feature name.", feature)
+        if '%' in feature:
+            logger.warning("Ignoring invalid feature '%s'. Feature name cannot contain the '%%' character.", feature)
             return False
     else:
         logger.warning("Ignoring invalid feature ''. Feature name cannot be empty.")
@@ -120,6 +124,42 @@ def validate_import_feature(feature):
 
 # File <-> List of KeyValue object
 
+def __read_with_appropriate_encoding(file_path, format_):
+    config_data = {}
+    default_encoding = 'utf-8'
+    detected_encoding = __check_file_encoding(file_path)
+
+    try:
+        with io.open(file_path, 'r', encoding=default_encoding) as config_file:
+            if format_ == 'json':
+                config_data = json.load(config_file)
+
+            elif format_ == 'yaml':
+                for yaml_data in list(yaml.safe_load_all(config_file)):
+                    config_data.update(yaml_data)
+
+            elif format_ == 'properties':
+                config_data = javaproperties.load(config_file)
+                logger.debug("Importing feature flags from a properties file is not supported. If properties file contains feature flags, they will be imported as regular key-values.")
+
+    except (UnicodeDecodeError, json.JSONDecodeError):
+        if detected_encoding == default_encoding:
+            raise
+
+        with io.open(file_path, 'r', encoding=detected_encoding) as config_file:
+            if format_ == 'json':
+                config_data = json.load(config_file)
+
+            elif format_ == 'yaml':
+                for yaml_data in list(yaml.safe_load_all(config_file)):
+                    config_data.update(yaml_data)
+
+            elif format_ == 'properties':
+                config_data = javaproperties.load(config_file)
+                logger.debug("Importing feature flags from a properties file is not supported. If properties file contains feature flags, they will be imported as regular key-values.")
+
+    return config_data
+
 
 def __read_kv_from_file(file_path,
                         format_,
@@ -129,34 +169,21 @@ def __read_kv_from_file(file_path,
                         content_type=None):
     config_data = {}
     try:
-        with io.open(file_path, 'r', encoding=__check_file_encoding(file_path)) as config_file:
-            if format_ == 'json':
-                config_data = json.load(config_file)
-                for feature_management_keyword in FEATURE_MANAGEMENT_KEYWORDS:
-                    # delete all feature management sections in any name format.
-                    # If users have not skipped features, and there are multiple
-                    # feature sections, we will error out while reading features.
-                    if feature_management_keyword in config_data:
-                        del config_data[feature_management_keyword]
-
-            elif format_ == 'yaml':
-                for yaml_data in list(yaml.safe_load_all(config_file)):
-                    config_data.update(yaml_data)
-                for feature_management_keyword in FEATURE_MANAGEMENT_KEYWORDS:
-                    # delete all feature management sections in any name format.
-                    # If users have not skipped features, and there are multiple
-                    # feature sections, we will error out while reading features.
-                    if feature_management_keyword in config_data:
-                        del config_data[feature_management_keyword]
-
-            elif format_ == 'properties':
-                config_data = javaproperties.load(config_file)
-                logger.debug("Importing feature flags from a properties file is not supported. If properties file contains feature flags, they will be imported as regular key-values.")
-
-    except ValueError:
-        raise CLIError('The input is not a well formatted %s file.' % (format_))
+        config_data = __read_with_appropriate_encoding(file_path, format_)
+        if format_ in ('json', 'yaml'):
+            for feature_management_keyword in FEATURE_MANAGEMENT_KEYWORDS:
+                # delete all feature management sections in any name format.
+                # If users have not skipped features, and there are multiple
+                # feature sections, we will error out while reading features.
+                if feature_management_keyword in config_data:
+                    del config_data[feature_management_keyword]
+    except ValueError as ex:
+        raise CLIError('The input is not a well formatted %s file.\nException: %s' % (format_, ex))
+    except yaml.YAMLError as ex:
+        raise CLIError('The input is not a well formatted YAML file.\nException: %s' % (ex))
     except OSError:
         raise CLIError('File is not available.')
+
     flattened_data = {}
     if format_ == 'json' and content_type and __is_json_content_type(content_type):
         for key in config_data:
@@ -202,17 +229,10 @@ def __read_features_from_file(file_path, format_):
         return features_dict
 
     try:
-        with io.open(file_path, 'r', encoding=__check_file_encoding(file_path)) as config_file:
-            if format_ == 'json':
-                config_data = json.load(config_file)
-
-            elif format_ == 'yaml':
-                for yaml_data in list(yaml.safe_load_all(config_file)):
-                    config_data.update(yaml_data)
-
+        config_data = __read_with_appropriate_encoding(file_path, format_)
         found_feature_section = False
         for index, feature_management_keyword in enumerate(FEATURE_MANAGEMENT_KEYWORDS):
-            # find the first occurence of feature management section in file.
+            # find the first occurrence of feature management section in file.
             # Enforce the same naming convention for 'EnabledFor' keyword
             # If there are multiple feature sections, we will error out here.
             if feature_management_keyword in config_data:
@@ -223,9 +243,11 @@ def __read_features_from_file(file_path, format_):
                 else:
                     raise CLIError('Unable to proceed because file contains multiple sections corresponding to "Feature Management".')
 
-    except ValueError:
+    except ValueError as ex:
         raise CLIError(
-            'The feature management section of input is not a well formatted %s file.' % (format_))
+            'The feature management section of input is not a well formatted %s file.\nException: %s' % (format_, ex))
+    except yaml.YAMLError as ex:
+        raise CLIError('The feature management section of input is not a well formatted YAML file.\nException: %s' % (ex))
     except OSError:
         raise CLIError('File is not available.')
 
@@ -244,7 +266,7 @@ def __write_kv_and_features_to_file(file_path, key_values=None, features=None, f
             if format_ == 'json':
                 json.dump(exported_keyvalues, fp, indent=2, ensure_ascii=False)
             elif format_ == 'yaml':
-                yaml.safe_dump(exported_keyvalues, fp, sort_keys=False)
+                yaml.safe_dump(exported_keyvalues, fp, sort_keys=False, width=float('inf'))
             elif format_ == 'properties':
                 javaproperties.dump(exported_keyvalues, fp)
     except Exception as exception:
@@ -298,7 +320,7 @@ def __read_kv_from_config_store(azconfig_client,
 
     if cli_ctx:
         from azure.cli.command_modules.keyvault._client_factory import keyvault_data_plane_factory
-        keyvault_client = keyvault_data_plane_factory(cli_ctx, None)
+        keyvault_client = keyvault_data_plane_factory(cli_ctx)
     else:
         keyvault_client = None
 
@@ -315,7 +337,7 @@ def __read_kv_from_config_store(azconfig_client,
 
             if kv.content_type and kv.value:
                 # resolve key vault reference
-                if keyvault_client and kv.content_type == KeyVaultConstants.KEYVAULT_CONTENT_TYPE:
+                if keyvault_client and __is_key_vault_ref(kv):
                     __resolve_secret(keyvault_client, kv)
 
         # trim unwanted fields from kv object instead of leaving them as null.
@@ -350,18 +372,11 @@ def __write_kv_and_features_to_config_store(azconfig_client,
         if not preserve_labels:
             set_kv.label = label
 
-        # Don't overwrite the content type of feature flags
-        if content_type and not __is_feature_flag(set_kv):
+        # Don't overwrite the content type of feature flags or key vault references
+        if content_type and not __is_feature_flag(set_kv) and not __is_key_vault_ref(set_kv):
             set_kv.content_type = content_type
 
-        try:
-            azconfig_client.set_configuration_setting(set_kv)
-        except ResourceReadOnlyError:
-            logger.warning("Failed to set read only key-value with key '%s' and label '%s'. Unlock the key-value before updating it.", set_kv.key, set_kv.label)
-        except HttpResponseError as exception:
-            logger.warning("Failed to set key-value with key '%s' and label '%s'. %s", set_kv.key, set_kv.label, str(exception))
-        except Exception as exception:
-            raise CLIError(str(exception))
+        __write_configuration_setting_to_config_store(azconfig_client, set_kv)
 
 
 def __is_feature_flag(kv):
@@ -370,6 +385,10 @@ def __is_feature_flag(kv):
     return False
 
 
+def __is_key_vault_ref(kv):
+    return kv and kv.content_type and kv.content_type.lower() == KeyVaultConstants.KEYVAULT_CONTENT_TYPE
+
+
 def __discard_features_from_retrieved_kv(src_kvs):
     try:
         src_kvs[:] = [kv for kv in src_kvs if not __is_feature_flag(kv)]
@@ -409,7 +428,6 @@ def __read_kv_from_app_service(cmd, appservice_account, prefix_to_add="", conten
                             secret_version = appsvc_value_dict.get('secretversion')
                             secret_identifier = "https://{0}.vault.azure.net/secrets/{1}/{2}".format(vault_name, secret_name, secret_version)
                         try:
-                            from azure.keyvault.key_vault_id import KeyVaultIdentifier
                             # this throws an exception for invalid format of secret identifier
                             KeyVaultIdentifier(uri=secret_identifier)
                             kv = KeyValue(key=key,
@@ -448,7 +466,7 @@ def __write_kv_to_app_service(cmd, key_values, appservice_account):
             name = kv.key
             value = kv.value
             # If its a KeyVault ref, convert the format to AppService KeyVault ref format
-            if kv.content_type and kv.content_type.lower() == KeyVaultConstants.KEYVAULT_CONTENT_TYPE:
+            if __is_key_vault_ref(kv):
                 try:
                     secret_uri = json.loads(value).get("uri")
                     if secret_uri:
@@ -529,21 +547,27 @@ def __serialize_feature_list_to_comparable_json_object(features):
         feature_json['description'] = feature.description
         # conditions
         feature_json['conditions'] = feature.conditions
-        # key
-        res[feature.key] = feature_json
+        # name
+        res[feature.name] = feature_json
     return res
 
 
-def __serialize_kv_list_to_comparable_json_list(keyvalues):
+def __serialize_kv_list_to_comparable_json_list(keyvalues, profile=None):
     res = []
     for kv in keyvalues:
         # value
-        kv_json = {'key': kv.key,
-                   'value': kv.value,
-                   'label': kv.label,
-                   'locked': kv.read_only,
-                   'last modified': kv.last_modified,
-                   'content type': kv.content_type}
+        if profile == ImportExportProfiles.KVSET:
+            kv_json = {'key': kv.key,
+                       'value': kv.value,
+                       'label': kv.label,
+                       'content_type': kv.content_type}
+        else:
+            kv_json = {'key': kv.key,
+                       'value': kv.value,
+                       'label': kv.label,
+                       'locked': kv.locked,
+                       'last modified': kv.last_modified,
+                       'content type': kv.content_type}
         # tags
         tag_json = {}
         if kv.tags:
@@ -644,6 +668,24 @@ def __print_preview(old_json, new_json):
     return True
 
 
+def __export_kvset_to_file(file_path, keyvalues, yes):
+    kvset = __serialize_kv_list_to_comparable_json_list(keyvalues, ImportExportProfiles.KVSET)
+    obj = {KVSetConstants.KVSETRootElementName: kvset}
+    json_string = json.dumps(obj, indent=2, ensure_ascii=False)
+    if not yes:
+        logger.warning('\n---------------- KVSet Preview (Beta) ----------------')
+        if len(kvset) == 0:
+            logger.warning('\nSource configuration is empty. Nothing to export.')
+            return
+        __print_preview_json_diff(new_obj=obj)
+        user_confirmation('Do you want to continue? \n')
+    try:
+        with open(file_path, 'w', encoding='utf-8') as fp:
+            fp.write(json_string)
+    except Exception as exception:
+        raise FileOperationError("Failed to export key-values to file. " + str(exception))
+
+
 def __print_restore_preview(kvs_to_restore, kvs_to_modify, kvs_to_delete):
     logger.warning('\n---------------- Preview (Beta) ----------------')
     if len(kvs_to_restore) + len(kvs_to_modify) + len(kvs_to_delete) == 0:
@@ -854,7 +896,7 @@ def __export_features(retrieved_features, naming_convention):
                         feature_filter["Parameters"] = filter_.parameters
                     feature_state[feature_reserved_keywords.enabledfor].append(feature_filter)
 
-            feature_entry = {feature.key: feature_state}
+            feature_entry = {feature.name: feature_state}
 
             exported_dict[feature_reserved_keywords.featuremanagement].update(feature_entry)
 
@@ -980,6 +1022,121 @@ def __resolve_secret(keyvault_client, keyvault_reference):
         raise CLIError(str(exception))
 
 
+def __import_kvset_from_file(client, path, yes):
+    new_kvset = __read_with_appropriate_encoding(file_path=path, format_='json')
+    if KVSetConstants.KVSETRootElementName not in new_kvset:
+        raise FileOperationError("file '{0}' is not in a valid '{1}' format.".format(path, ImportExportProfiles.KVSET))
+
+    kvset_to_import = [ConfigurationSetting(key=kv['key'],
+                                            label=kv['label'],
+                                            content_type=kv['content_type'],
+                                            value=kv['value'],
+                                            tags=kv['tags'])
+                       for kv in new_kvset[KVSetConstants.KVSETRootElementName]]
+
+    if not yes:
+        existing_kvset = __read_kv_from_config_store(client,
+                                                     key=SearchFilterOptions.ANY_KEY,
+                                                     label=SearchFilterOptions.ANY_LABEL)
+        # we don't delete configurations if they are missing from the import file, so don't need to show them in the
+        # diff, so omit them from existing kvset
+        existing_kvset = list(filter(lambda kv:
+                                     any(kv_import.key == kv.key and kv_import.label == kv.label
+                                         for kv_import in kvset_to_import), existing_kvset))
+
+        existing_kvset_list = __serialize_kv_list_to_comparable_json_list(existing_kvset, ImportExportProfiles.KVSET)
+        kvset_to_import_list = __serialize_kv_list_to_comparable_json_list(kvset_to_import, ImportExportProfiles.KVSET)
+
+        logger.warning('\n---------------- KVSet Preview (Beta) ----------------')
+        changes_detected = __print_preview_json_diff(existing_kvset_list, kvset_to_import_list)
+        if not changes_detected:
+            logger.warning('Target configuration store already contains all configuration settings in source. No changes will be made.')
+            return
+
+        user_confirmation('Do you want to continue?\n')
+
+    for config_setting in kvset_to_import:
+        if __is_key_vault_ref(kv=config_setting):
+            if not __validate_import_keyvault_ref(kv=config_setting):
+                continue
+        elif __is_feature_flag(kv=config_setting):
+            if not __validate_import_feature_flag(kv=config_setting):
+                continue
+        elif not validate_import_key(config_setting.key):
+            continue
+
+        # All validations successful
+        __write_configuration_setting_to_config_store(client, config_setting)
+
+
+def __validate_import_keyvault_ref(kv):
+    if kv and validate_import_key(kv.key):
+        try:
+            value = json.loads(kv.value)
+        except JSONDecodeError as exception:
+            logger.warning("The keyvault reference with key '{%s}' is not in a valid JSON format. It will not be imported.\n{%s}", kv.key, str(exception))
+            return False
+
+        if 'uri' in value:
+            parsed_url = urlparse(value['uri'])
+            # URL with a valid scheme and netloc is a valid url, but keyvault ref has path as well, so validate it
+            if parsed_url.scheme and parsed_url.netloc and parsed_url.path:
+                try:
+                    KeyVaultIdentifier(uri=value['uri'])
+                    return True
+                except Exception:  # pylint: disable=broad-except
+                    pass
+
+        logger.warning("Keyvault reference with key '{%s}' is not a valid keyvault reference. It will not be imported.", kv.key)
+    return False
+
+
+def __validate_import_feature_flag(kv):
+    if kv and validate_import_feature(kv.key):
+        try:
+            ff = json.loads(kv.value)
+            if ff['id'] and ff['description'] and ff['enabled'] and ff['conditions']:
+                return True
+            logger.warning("The feature flag with key '{%s}' is not a valid feature flag. It will not be imported.", kv.key)
+        except JSONDecodeError as exception:
+            logger.warning("The feature flag with key '{%s}' is not in a valid JSON format. It will not be imported.\n{%s}", kv.id, str(exception))
+    return False
+
+
+def __write_configuration_setting_to_config_store(azconfig_client, configuration_setting):
+    try:
+        azconfig_client.set_configuration_setting(configuration_setting)
+    except ResourceReadOnlyError:
+        logger.warning(
+            "Failed to set read only key-value with key '%s' and label '%s'. Unlock the key-value before updating it.",
+            configuration_setting.key, configuration_setting.label)
+    except HttpResponseError as exception:
+        logger.warning(
+            "Failed to set key-value with key '%s' and label '%s'. %s",
+            configuration_setting.key, configuration_setting.label, str(exception))
+    except Exception as exception:
+        raise AzureInternalError(str(exception))
+
+
+def __print_preview_json_diff(old_obj=None, new_obj=None):
+    # prints the json diff if two objects differ, returns whether the diff was found.
+
+    old_json = "" if old_obj is None else json.dumps(old_obj, indent=2, ensure_ascii=False).splitlines(True)
+    new_json = "" if new_obj is None else json.dumps(new_obj, indent=2, ensure_ascii=False).splitlines(True)
+
+    differ = Differ()
+    diff = list(differ.compare(old_json, new_json))
+
+    if not any(line.startswith('-') or line.startswith('+') for line in diff):
+        return False
+
+    # omit minuscule details of the diff outlining the characters that changed, and show rest of the diff.
+    logger.warning(''.join(filter(lambda line: not line.startswith('?'), diff)))
+    # print newline for readability
+    logger.warning('\n')
+    return True
+
+
 class Undef:  # pylint: disable=too-few-public-methods
     '''
     Dummy undef class used to preallocate space for kv exporting.
