@@ -7,22 +7,25 @@
 from azure.mgmt.network import NetworkManagementClient
 from azure.mgmt.web import WebSiteManagementClient
 from azure.mgmt.resource import ResourceManagementClient
+from azure.mgmt.privatedns import PrivateDnsManagementClient
 
 # Models
-from azure.mgmt.network.models import (RouteTable, Route, NetworkSecurityGroup, SecurityRule)
-from azure.mgmt.resource.resources.models import (DeploymentProperties, Deployment)
+from azure.mgmt.network.models import (RouteTable, Route, NetworkSecurityGroup, SecurityRule, Delegation)
+from azure.mgmt.resource.resources.models import (DeploymentProperties, Deployment, SubResource)
+from azure.mgmt.privatedns.models import (PrivateZone, VirtualNetworkLink, RecordSet, ARecord)
 
 # Utils
 from azure.cli.core.commands import LongRunningOperation
 from azure.cli.core.commands.client_factory import (get_mgmt_service_client, get_subscription_id)
 from azure.cli.core.commands.arm import ArmTemplateBuilder
 from azure.cli.core.util import (sdk_no_wait, random_string)
+from azure.cli.core.azclierror import (ResourceNotFoundError, ValidationError, CommandNotFoundError,
+                                       MutuallyExclusiveArgumentError)
 from knack.log import get_logger
-from knack.util import CLIError
 from msrestazure.tools import (parse_resource_id, is_valid_resource_id, resource_id)
 
+VERSION_2019_08_01 = "2019-08-01"
 VERSION_2019_10_01 = "2019-10-01"
-VERSION_2020_04_01 = "2020-04-01"
 
 logger = get_logger(__name__)
 
@@ -41,11 +44,12 @@ def show_appserviceenvironment(cmd, name, resource_group_name=None):
     return ase_client.get(resource_group_name, name)
 
 
-def create_appserviceenvironment_arm(cmd, resource_group_name, name, subnet, vnet_name=None, ignore_route_table=False,
+def create_appserviceenvironment_arm(cmd, resource_group_name, name, subnet, kind='ASEv2',
+                                     vnet_name=None, ignore_route_table=False,
                                      ignore_network_security_group=False, virtual_ip_type='Internal',
                                      front_end_scale_factor=None, front_end_sku=None, force_route_table=False,
                                      force_network_security_group=False, ignore_subnet_size_validation=False,
-                                     location=None, no_wait=False):
+                                     location=None, no_wait=False, os_preference=None, zone_redundant=None):
     # The current SDK has a couple of challenges creating ASE. The current swagger version used,
     # did not have 201 as valid response code, and thus will fail with polling operations.
     # The Load Balancer Type is an Enum Flag, that is expressed as a simple string enum in swagger,
@@ -53,25 +57,33 @@ def create_appserviceenvironment_arm(cmd, resource_group_name, name, subnet, vne
     # Therefore the current method use direct ARM.
     location = location or _get_location_from_resource_group(cmd.cli_ctx, resource_group_name)
     subnet_id = _validate_subnet_id(cmd.cli_ctx, subnet, vnet_name, resource_group_name)
-
+    deployment_name = _get_unique_deployment_name('cli_ase_deploy_')
     _validate_subnet_empty(cmd.cli_ctx, subnet_id)
     if not ignore_subnet_size_validation:
         _validate_subnet_size(cmd.cli_ctx, subnet_id)
-    if not ignore_route_table:
-        _ensure_route_table(cmd.cli_ctx, resource_group_name, name, location, subnet_id, force_route_table)
-    if not ignore_network_security_group:
-        _ensure_network_security_group(cmd.cli_ctx, resource_group_name, name, location,
-                                       subnet_id, force_network_security_group)
 
+    if kind == 'ASEv2':
+        if not ignore_route_table:
+            _ensure_route_table(cmd.cli_ctx, resource_group_name, name, location, subnet_id, force_route_table)
+        if not ignore_network_security_group:
+            _ensure_network_security_group(cmd.cli_ctx, resource_group_name, name, location,
+                                           subnet_id, force_network_security_group)
+        ase_deployment_properties = _build_ase_deployment_properties(name=name, location=location,
+                                                                     subnet_id=subnet_id,
+                                                                     virtual_ip_type=virtual_ip_type,
+                                                                     front_end_scale_factor=front_end_scale_factor,
+                                                                     front_end_sku=front_end_sku,
+                                                                     os_preference=os_preference)
+
+    elif kind == 'ASEv3':
+        _ensure_subnet_delegation(cmd.cli_ctx, subnet_id, 'Microsoft.Web/hostingEnvironments')
+        ase_deployment_properties = _build_ase_deployment_properties(name=name, location=location,
+                                                                     subnet_id=subnet_id, kind='ASEv3',
+                                                                     virtual_ip_type=virtual_ip_type,
+                                                                     zone_redundant=zone_redundant)
     logger.info('Create App Service Environment...')
-    deployment_name = _get_unique_deployment_name('cli_ase_deploy_')
-    ase_deployment_properties = _build_ase_deployment_properties(name=name, location=location,
-                                                                 subnet_id=subnet_id, virtual_ip_type=virtual_ip_type,
-                                                                 front_end_scale_factor=front_end_scale_factor,
-                                                                 front_end_sku=front_end_sku, tags=None)
-
     deployment_client = _get_resource_client_factory(cmd.cli_ctx).deployments
-    return sdk_no_wait(no_wait, deployment_client.create_or_update,
+    return sdk_no_wait(no_wait, deployment_client.begin_create_or_update,
                        resource_group_name, deployment_name, ase_deployment_properties)
 
 
@@ -80,7 +92,7 @@ def delete_appserviceenvironment(cmd, name, resource_group_name=None, no_wait=Fa
     if resource_group_name is None:
         resource_group_name = _get_resource_group_name_from_ase(ase_client, name)
 
-    return sdk_no_wait(no_wait, ase_client.delete,
+    return sdk_no_wait(no_wait, ase_client.begin_delete,
                        resource_group_name=resource_group_name, name=name)
 
 
@@ -90,6 +102,9 @@ def update_appserviceenvironment(cmd, name, resource_group_name=None, front_end_
     if resource_group_name is None:
         resource_group_name = _get_resource_group_name_from_ase(ase_client, name)
     ase_def = ase_client.get(resource_group_name, name)
+    if ase_def.kind.lower() != 'asev2':
+        raise CommandNotFoundError('Only ASEv2 currently supports update')
+
     worker_sku = _map_worker_sku(front_end_sku)
     ase_def.worker_pools = ase_def.worker_pools or []  # v1 feature, but cannot be null
     ase_def.internal_load_balancing_mode = None  # Workaround issue with flag enums in Swagger
@@ -98,7 +113,7 @@ def update_appserviceenvironment(cmd, name, resource_group_name=None, front_end_
     if front_end_scale_factor:
         ase_def.front_end_scale_factor = front_end_scale_factor
 
-    return sdk_no_wait(no_wait, ase_client.create_or_update, resource_group_name=resource_group_name,
+    return sdk_no_wait(no_wait, ase_client.begin_create_or_update, resource_group_name=resource_group_name,
                        name=name, hosting_environment_envelope=ase_def)
 
 
@@ -106,6 +121,10 @@ def list_appserviceenvironment_addresses(cmd, name, resource_group_name=None):
     ase_client = _get_ase_client_factory(cmd.cli_ctx)
     if resource_group_name is None:
         resource_group_name = _get_resource_group_name_from_ase(ase_client, name)
+    ase = ase_client.get(resource_group_name, name)
+    if ase.kind.lower() == 'asev3':
+        # return ase_client.get_ase_v3_networking_configuration(resource_group_name, name) # pending SDK update
+        raise CommandNotFoundError('list-addresses is currently not supported for ASEv3.')
     return ase_client.get_vip_info(resource_group_name, name)
 
 
@@ -116,8 +135,37 @@ def list_appserviceenvironment_plans(cmd, name, resource_group_name=None):
     return ase_client.list_app_service_plans(resource_group_name, name)
 
 
-def _get_ase_client_factory(cli_ctx):
+def create_ase_inbound_services(cmd, resource_group_name, name, subnet, vnet_name=None, skip_dns=False):
+    ase_client = _get_ase_client_factory(cmd.cli_ctx)
+    ase = ase_client.get(resource_group_name, name)
+    if not ase:
+        raise ResourceNotFoundError("App Service Environment '{}' not found.".format(name))
+
+    if ase.internal_load_balancing_mode == 'None':
+        raise ValidationError('Private DNS Zone is not relevant for External ASE.')
+
+    if ase.kind.lower() == 'asev3':
+        # pending SDK update (ase_client.get_ase_v3_networking_configuration(resource_group_name, name))
+        raise CommandNotFoundError('create-inbound-services is currently not supported for ASEv3.')
+
+    ase_vip_info = ase_client.get_vip_info(resource_group_name, name)
+    inbound_ip_address = ase_vip_info.internal_ip_address
+    inbound_subnet_id = _validate_subnet_id(cmd.cli_ctx, subnet, vnet_name, resource_group_name)
+    inbound_vnet_id = _get_vnet_id_from_subnet(cmd.cli_ctx, inbound_subnet_id)
+
+    if not skip_dns:
+        _ensure_ase_private_dns_zone(cmd.cli_ctx, resource_group_name=resource_group_name, name=name,
+                                     inbound_vnet_id=inbound_vnet_id, inbound_ip_address=inbound_ip_address)
+    else:
+        logger.warning('Parameter --skip-dns is deprecated.')
+
+
+def _get_ase_client_factory(cli_ctx, api_version=None):
     client = get_mgmt_service_client(cli_ctx, WebSiteManagementClient).app_service_environments
+    if api_version:
+        client.api_version = api_version
+    else:
+        client.api_version = VERSION_2019_08_01
     return client
 
 
@@ -126,16 +174,22 @@ def _get_resource_client_factory(cli_ctx, api_version=None):
     if api_version:
         client.api_version = api_version
     else:
-        api_version = VERSION_2019_10_01
+        client.api_version = VERSION_2019_10_01
     return client
 
 
 def _get_network_client_factory(cli_ctx, api_version=None):
+    from azure.cli.core.profiles import AD_HOC_API_VERSIONS, ResourceType
     client = get_mgmt_service_client(cli_ctx, NetworkManagementClient)
     if api_version:
         client.api_version = api_version
     else:
-        api_version = VERSION_2020_04_01
+        client.api_version = AD_HOC_API_VERSIONS[ResourceType.MGMT_NETWORK]['appservice_network']
+    return client
+
+
+def _get_private_dns_client_factory(cli_ctx):
+    client = get_mgmt_service_client(cli_ctx, PrivateDnsManagementClient)
     return client
 
 
@@ -155,7 +209,7 @@ def _get_resource_group_name_from_ase(ase_client, ase_name):
             ase_found = True
             break
     if not ase_found:
-        raise CLIError("App service environment '{}' not found in subscription.".format(ase_name))
+        raise ResourceNotFoundError("App Service Environment '{}' not found in subscription.".format(ase_name))
     return resource_group
 
 
@@ -172,7 +226,19 @@ def _validate_subnet_id(cli_ctx, subnet, vnet_name, resource_group_name):
             name=vnet_name,
             child_type_1='subnets',
             child_name_1=subnet)
-    raise CLIError('Usage error: --subnet ID | --subnet NAME --vnet-name NAME')
+    raise MutuallyExclusiveArgumentError('Please specify either: --subnet ID or (--subnet NAME and --vnet-name NAME)')
+
+
+def _get_vnet_id_from_subnet(cli_ctx, subnet_id):
+    subnet_id_parts = parse_resource_id(subnet_id)
+    vnet_resource_group = subnet_id_parts['resource_group']
+    vnet_name = subnet_id_parts['name']
+    return resource_id(
+        subscription=get_subscription_id(cli_ctx),
+        resource_group=vnet_resource_group,
+        namespace='Microsoft.Network',
+        type='virtualNetworks',
+        name=vnet_name)
 
 
 def _map_worker_sku(sku_name):
@@ -191,8 +257,8 @@ def _validate_subnet_empty(cli_ctx, subnet_id):
     subnet_name = subnet_id_parts['resource_name']
     network_client = _get_network_client_factory(cli_ctx)
     subnet_obj = network_client.subnets.get(vnet_resource_group, vnet_name, subnet_name)
-    if subnet_obj.resource_navigation_links:
-        raise CLIError('Subnet is not empty.')
+    if subnet_obj.resource_navigation_links or subnet_obj.service_association_links:
+        raise ValidationError('Subnet is not empty.')
 
 
 def _validate_subnet_size(cli_ctx, subnet_id):
@@ -205,7 +271,39 @@ def _validate_subnet_size(cli_ctx, subnet_id):
     address = subnet_obj.address_prefix
     size = int(address[address.index('/') + 1:])
     if size > 24:
-        raise CLIError('Subnet is too small. Should be at least /24.')
+        err_msg = 'Subnet size could cause scaling issues. Recommended size is at least /24.'
+        rec_msg = 'Use --ignore-subnet-size-validation to skip size test.'
+        validation_error = ValidationError(err_msg)
+        validation_error.set_recommendation(rec_msg)
+        raise validation_error
+
+
+def _ensure_subnet_delegation(cli_ctx, subnet_id, delegation_service_name):
+    network_client = _get_network_client_factory(cli_ctx)
+    subnet_id_parts = parse_resource_id(subnet_id)
+    vnet_resource_group = subnet_id_parts['resource_group']
+    vnet_name = subnet_id_parts['name']
+    subnet_name = subnet_id_parts['resource_name']
+    subnet_obj = network_client.subnets.get(vnet_resource_group, vnet_name, subnet_name)
+
+    delegations = subnet_obj.delegations
+    delegated = False
+    for d in delegations:
+        if d.service_name.lower() == delegation_service_name.lower():
+            delegated = True
+
+    if not delegated:
+        subnet_obj.delegations = [Delegation(name="delegation", service_name=delegation_service_name)]
+        try:
+            poller = network_client.subnets.begin_create_or_update(
+                vnet_resource_group, vnet_name, subnet_name, subnet_parameters=subnet_obj)
+            LongRunningOperation(cli_ctx)(poller)
+        except Exception:
+            err_msg = 'Subnet must be delegated to {}.'.format(delegation_service_name)
+            rec_msg = 'Use: az network vnet subnet update --delegations "{}"'.format(delegation_service_name)
+            validation_error = ValidationError(err_msg)
+            validation_error.set_recommendation(rec_msg)
+            raise validation_error
 
 
 def _ensure_route_table(cli_ctx, resource_group_name, ase_name, location, subnet_id, force):
@@ -251,9 +349,12 @@ def _ensure_route_table(cli_ctx, resource_group_name, ase_name, location, subnet
         route_table_id_parts = parse_resource_id(subnet_obj.route_table.id)
         rt_name = route_table_id_parts['name']
         if rt_name.lower() != ase_route_table_name.lower():
-            raise CLIError('Route table already exists. \
-                            Use --ignore-route-table to use existing route table. \
-                            Use --force-route-table to replace existing route table')
+            err_msg = 'Route table already exists.'
+            rec_msg = 'Use --ignore-route-table to use existing route table ' \
+                      'or --force-route-table to replace existing route table'
+            validation_error = ValidationError(err_msg)
+            validation_error.set_recommendation(rec_msg)
+            raise validation_error
 
 
 def _ensure_network_security_group(cli_ctx, resource_group_name, ase_name, location, subnet_id, force):
@@ -281,79 +382,33 @@ def _ensure_network_security_group(cli_ctx, resource_group_name, ase_name, locat
                                                                                    ase_nsg_name, ase_nsg)
             LongRunningOperation(cli_ctx)(poller)
 
-            logger.info('Ensure Network Security Group Rules...')
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-management',
-                             100, 'Used to manage ASE from public VIP', '*', 'Allow', 'Inbound',
-                             '*', 'AppServiceManagement', '454-455', '*')
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-load-balancer-keep-alive',
-                             105, 'Allow communication to ASE from Load Balancer', '*', 'Allow', 'Inbound',
-                             '*', 'AzureLoadBalancer', '16001', '*')
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'ASE-internal-inbound',
-                             110, 'ASE-internal-inbound', '*', 'Allow', 'Inbound',
-                             '*', subnet_address_prefix, '*', '*')
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-HTTP',
-                             120, 'Allow HTTP', '*', 'Allow', 'Inbound',
-                             '*', '*', '80', '*')
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-HTTPS',
-                             130, 'Allow HTTPS', '*', 'Allow', 'Inbound',
-                             '*', '*', '443', '*')
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-FTP',
-                             140, 'Allow FTP', '*', 'Allow', 'Inbound',
-                             '*', '*', '21', '*')
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-FTPS',
-                             150, 'Allow FTPS', '*', 'Allow', 'Inbound',
-                             '*', '*', '990', '*')
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-FTP-Data',
-                             160, 'Allow FTP Data', '*', 'Allow', 'Inbound',
-                             '*', '*', '10001-10020', '*')
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-Remote-Debugging',
-                             170, 'Visual Studio remote debugging', '*', 'Allow', 'Inbound',
-                             '*', '*', '4016-4022', '*')
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Outbound-443',
-                             100, 'Azure Storage blob', '*', 'Allow', 'Outbound',
-                             '*', '*', '443', '*')
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Outbound-DB',
-                             110, 'Database', '*', 'Allow', 'Outbound',
-                             '*', '*', '1433', 'Sql')
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Outbound-DNS',
-                             120, 'DNS', '*', 'Allow', 'Outbound',
-                             '*', '*', '53', '*')
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'ASE-internal-outbound',
-                             130, 'Azure Storage queue', '*', 'Allow', 'Outbound',
-                             '*', '*', '*', subnet_address_prefix)
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Outbound-80',
-                             140, 'Outbound 80', '*', 'Allow', 'Outbound',
-                             '*', '*', '80', '*')
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Outbound-monitor',
-                             150, 'Azure Monitor', '*', 'Allow', 'Outbound',
-                             '*', '*', 12000, '*')
-            _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Outbound-NTP',
-                             160, 'Clock', '*', 'Allow', 'Outbound',
-                             '*', '*', '123', '*')
+            _create_asev2_nsg_rules(cli_ctx, resource_group_name, ase_nsg_name, subnet_address_prefix)
 
         nsg = network_client.network_security_groups.get(resource_group_name, ase_nsg_name)
         if not subnet_obj.network_security_group or subnet_obj.network_security_group.id != nsg.id:
             logger.info('Associate Network Security Group with Subnet...')
             subnet_obj.network_security_group = NetworkSecurityGroup(id=nsg.id)
             poller = network_client.subnets.begin_create_or_update(
-                vnet_resource_group, vnet_name,
-                subnet_name, subnet_parameters=subnet_obj)
+                vnet_resource_group, vnet_name, subnet_name, subnet_parameters=subnet_obj)
             LongRunningOperation(cli_ctx)(poller)
     else:
         nsg_id_parts = parse_resource_id(subnet_obj.network_security_group.id)
         nsg_name = nsg_id_parts['name']
         if nsg_name.lower() != ase_nsg_name.lower():
-            raise CLIError('Network Security Group already exists. \
-                            Use --ignore-network-security-group to use existing NSG. \
-                            Use --force-network-security-group to replace existing NSG')
+            err_msg = 'Network Security Group already exists.'
+            rec_msg = 'Use --ignore-network-security-group to use existing NSG ' \
+                      'or --force-network-security-group to replace existing NSG'
+            validation_error = ValidationError(err_msg, rec_msg)
+            raise validation_error
 
 
 def _get_unique_deployment_name(prefix):
     return prefix + random_string(16)
 
 
-def _build_ase_deployment_properties(name, location, subnet_id, virtual_ip_type,
-                                     front_end_scale_factor=None, front_end_sku=None, tags=None):
+def _build_ase_deployment_properties(name, location, subnet_id, virtual_ip_type=None,
+                                     front_end_scale_factor=None, front_end_sku=None, tags=None,
+                                     kind='ASEv2', os_preference=None, zone_redundant=None):
     # InternalLoadBalancingMode Enum: None 0, Web 1, Publishing 2.
     # External: 0 (None), Internal: 3 (Web + Publishing)
     ilb_mode = 3 if virtual_ip_type == 'Internal' else 0
@@ -362,7 +417,7 @@ def _build_ase_deployment_properties(name, location, subnet_id, virtual_ip_type,
         'location': location,
         'InternalLoadBalancingMode': ilb_mode,
         'virtualNetwork': {
-            'Id': subnet_id
+            'id': subnet_id
         }
     }
     if front_end_scale_factor:
@@ -370,13 +425,17 @@ def _build_ase_deployment_properties(name, location, subnet_id, virtual_ip_type,
     if front_end_sku:
         worker_sku = _map_worker_sku(front_end_sku)
         ase_properties['multiSize'] = worker_sku
+    if os_preference:
+        ase_properties['osPreference'] = os_preference
+    if zone_redundant:
+        ase_properties['zoneRedundant'] = zone_redundant
 
     ase_resource = {
         'name': name,
         'type': 'Microsoft.Web/hostingEnvironments',
         'location': location,
-        'apiVersion': '2019-02-01',
-        'kind': 'ASEV2',
+        'apiVersion': '2019-08-01',
+        'kind': kind,
         'tags': tags,
         'properties': ase_properties
     }
@@ -391,6 +450,58 @@ def _build_ase_deployment_properties(name, location, subnet_id, virtual_ip_type,
     return deployment
 
 
+def _create_asev2_nsg_rules(cli_ctx, resource_group_name, ase_nsg_name, subnet_address_prefix):
+    logger.info('Ensure Network Security Group Rules...')
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-management',
+                     100, 'Used to manage ASE from public VIP', '*', 'Allow', 'Inbound',
+                     '*', 'AppServiceManagement', '454-455', '*')
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-load-balancer-keep-alive',
+                     105, 'Allow communication to ASE from Load Balancer', '*', 'Allow', 'Inbound',
+                     '*', 'AzureLoadBalancer', '16001', '*')
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'ASE-internal-inbound',
+                     110, 'ASE-internal-inbound', '*', 'Allow', 'Inbound',
+                     '*', subnet_address_prefix, '*', '*')
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-HTTP',
+                     120, 'Allow HTTP', '*', 'Allow', 'Inbound',
+                     '*', '*', '80', '*')
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-HTTPS',
+                     130, 'Allow HTTPS', '*', 'Allow', 'Inbound',
+                     '*', '*', '443', '*')
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-FTP',
+                     140, 'Allow FTP', '*', 'Allow', 'Inbound',
+                     '*', '*', '21', '*')
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-FTPS',
+                     150, 'Allow FTPS', '*', 'Allow', 'Inbound',
+                     '*', '*', '990', '*')
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-FTP-Data',
+                     160, 'Allow FTP Data', '*', 'Allow', 'Inbound',
+                     '*', '*', '10001-10020', '*')
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Inbound-Remote-Debugging',
+                     170, 'Visual Studio remote debugging', '*', 'Allow', 'Inbound',
+                     '*', '*', '4016-4022', '*')
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Outbound-443',
+                     100, 'Azure Storage blob', '*', 'Allow', 'Outbound',
+                     '*', '*', '443', '*')
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Outbound-DB',
+                     110, 'Database', '*', 'Allow', 'Outbound',
+                     '*', '*', '1433', 'Sql')
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Outbound-DNS',
+                     120, 'DNS', '*', 'Allow', 'Outbound',
+                     '*', '*', '53', '*')
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'ASE-internal-outbound',
+                     130, 'Azure Storage queue', '*', 'Allow', 'Outbound',
+                     '*', '*', '*', subnet_address_prefix)
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Outbound-80',
+                     140, 'Outbound 80', '*', 'Allow', 'Outbound',
+                     '*', '*', '80', '*')
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Outbound-monitor',
+                     150, 'Azure Monitor', '*', 'Allow', 'Outbound',
+                     '*', '*', '12000', '*')
+    _create_nsg_rule(cli_ctx, resource_group_name, ase_nsg_name, 'Outbound-NTP',
+                     160, 'Clock', '*', 'Allow', 'Outbound',
+                     '*', '*', '123', '*')
+
+
 def _create_nsg_rule(cli_ctx, resource_group_name, network_security_group_name, security_rule_name,
                      priority, description=None, protocol=None, access=None, direction=None,
                      source_port_range='*', source_address_prefix='*',
@@ -406,3 +517,25 @@ def _create_nsg_rule(cli_ctx, resource_group_name, network_security_group_name,
     poller = network_client.security_rules.begin_create_or_update(
         resource_group_name, network_security_group_name, security_rule_name, settings)
     LongRunningOperation(cli_ctx)(poller)
+
+
+def _ensure_ase_private_dns_zone(cli_ctx, resource_group_name, name, inbound_vnet_id, inbound_ip_address):
+    # Private DNS Zone
+    private_dns_client = _get_private_dns_client_factory(cli_ctx)
+    zone_name = '{}.appserviceenvironment.net'.format(name)
+    zone = PrivateZone(location='global', tags=None)
+    poller = private_dns_client.private_zones.begin_create_or_update(resource_group_name, zone_name, zone)
+    LongRunningOperation(cli_ctx)(poller)
+
+    link_name = '{}_link'.format(name)
+    link = VirtualNetworkLink(location='global', tags=None)
+    link.virtual_network = SubResource(id=inbound_vnet_id)
+    link.registration_enabled = False
+    private_dns_client.virtual_network_links.begin_create_or_update(resource_group_name, zone_name,
+                                                                    link_name, link, if_none_match='*')
+    ase_record = ARecord(ipv4_address=inbound_ip_address)
+    record_set = RecordSet(ttl=3600)
+    record_set.a_records = [ase_record]
+    private_dns_client.record_sets.create_or_update(resource_group_name, zone_name, 'a', '*', record_set)
+    private_dns_client.record_sets.create_or_update(resource_group_name, zone_name, 'a', '@', record_set)
+    private_dns_client.record_sets.create_or_update(resource_group_name, zone_name, 'a', '*.scm', record_set)
