@@ -8,7 +8,7 @@ from azure.cli.core.commands import CliCommandType
 from azure.cli.core.util import empty_on_404
 
 from ._client_factory import cf_web_client, cf_plans, cf_webapps
-from ._validators import validate_app_exists_in_rg, validate_app_or_slot_exists_in_rg, validate_asp_sku
+from ._validators import validate_onedeploy_params
 
 
 def output_slots_in_table(slots):
@@ -110,25 +110,27 @@ def load_command_table(self, _):
 
     appservice_domains = CliCommandType(operations_tmpl='azure.cli.command_modules.appservice.appservice_domains#{}')
 
+    logicapp_custom = CliCommandType(operations_tmpl='azure.cli.command_modules.appservice.logicapp.custom#{}')
+
     with self.command_group('webapp', webapp_sdk) as g:
         g.custom_command('create', 'create_webapp', exception_handler=ex_handler_factory())
         g.custom_command('up', 'webapp_up', exception_handler=ex_handler_factory())
         g.custom_command('ssh', 'ssh_webapp', exception_handler=ex_handler_factory(), is_preview=True)
         g.custom_command('list', 'list_webapp', table_transformer=transform_web_list_output)
-        g.custom_show_command('show', 'show_webapp', table_transformer=transform_web_output,
-                              validator=validate_app_or_slot_exists_in_rg)
-        g.custom_command('delete', 'delete_webapp', validator=validate_app_or_slot_exists_in_rg)
-        g.custom_command('stop', 'stop_webapp', validator=validate_app_or_slot_exists_in_rg)
-        g.custom_command('start', 'start_webapp', validator=validate_app_or_slot_exists_in_rg)
-        g.custom_command('restart', 'restart_webapp', validator=validate_app_or_slot_exists_in_rg)
+        g.custom_show_command('show', 'show_webapp', table_transformer=transform_web_output)
+        g.custom_command('delete', 'delete_webapp')
+        g.custom_command('stop', 'stop_webapp')
+        g.custom_command('start', 'start_webapp')
+        g.custom_command('restart', 'restart_webapp')
         g.custom_command('browse', 'view_in_browser')
-        g.custom_command('list-instances', 'list_instances', validator=validate_app_or_slot_exists_in_rg)
-        # Move back to using list_runtimes function once Available Stacks API is updated (it's updated with Antares deployments)
+        g.custom_command('list-instances', 'list_instances')
+        # TO DO: Move back to using list_runtimes function once Available Stacks API is updated (it's updated with Antares deployments)
         g.custom_command('list-runtimes', 'list_runtimes_hardcoded')
-        g.custom_command('identity assign', 'assign_identity', validator=validate_app_or_slot_exists_in_rg)
-        g.custom_show_command('identity show', 'show_identity', validator=validate_app_or_slot_exists_in_rg)
-        g.custom_command('identity remove', 'remove_identity', validator=validate_app_or_slot_exists_in_rg)
+        g.custom_command('identity assign', 'assign_identity')
+        g.custom_show_command('identity show', 'show_identity')
+        g.custom_command('identity remove', 'remove_identity')
         g.custom_command('create-remote-connection', 'create_tunnel', exception_handler=ex_handler_factory())
+        g.custom_command('deploy', 'perform_onedeploy', validator=validate_onedeploy_params, is_preview=True)
         g.generic_update_command('update', getter_name='get_webapp', setter_name='set_webapp', custom_func_name='update_webapp', command_type=appservice_custom)
 
     with self.command_group('webapp traffic-routing') as g:
@@ -156,10 +158,10 @@ def load_command_table(self, _):
         g.custom_command('delete', 'delete_connection_strings')
 
     with self.command_group('webapp config storage-account') as g:
-        g.custom_command('list', 'get_azure_storage_accounts', exception_handler=empty_on_404, is_preview=True)
-        g.custom_command('add', 'add_azure_storage_account', is_preview=True)
-        g.custom_command('update', 'update_azure_storage_account', is_preview=True)
-        g.custom_command('delete', 'delete_azure_storage_accounts', is_preview=True)
+        g.custom_command('list', 'get_azure_storage_accounts', exception_handler=empty_on_404)
+        g.custom_command('add', 'add_azure_storage_account')
+        g.custom_command('update', 'update_azure_storage_account')
+        g.custom_command('delete', 'delete_azure_storage_accounts')
 
     with self.command_group('webapp config hostname') as g:
         g.custom_command('add', 'add_hostname', exception_handler=ex_handler_factory())
@@ -173,11 +175,11 @@ def load_command_table(self, _):
         g.custom_show_command('show', 'show_container_settings')
 
     with self.command_group('webapp config ssl') as g:
-        g.custom_command('upload', 'upload_ssl_cert', validator=validate_app_exists_in_rg)
+        g.custom_command('upload', 'upload_ssl_cert')
         g.custom_command('list', 'list_ssl_certs', exception_handler=ex_handler_factory())
         g.custom_show_command('show', 'show_ssl_cert', exception_handler=ex_handler_factory())
-        g.custom_command('bind', 'bind_ssl_cert', exception_handler=ex_handler_factory(), validator=validate_app_or_slot_exists_in_rg)
-        g.custom_command('unbind', 'unbind_ssl_cert', validator=validate_app_or_slot_exists_in_rg)
+        g.custom_command('bind', 'bind_ssl_cert', exception_handler=ex_handler_factory())
+        g.custom_command('unbind', 'unbind_ssl_cert')
         g.custom_command('delete', 'delete_ssl_cert', exception_handler=ex_handler_factory())
         g.custom_command('import', 'import_ssl_cert', exception_handler=ex_handler_factory())
         g.custom_command('create', 'create_managed_ssl_cert', exception_handler=ex_handler_factory(), is_preview=True)
@@ -215,10 +217,10 @@ def load_command_table(self, _):
         g.custom_command('update-token', 'update_git_token', exception_handler=ex_handler_factory())
 
     with self.command_group('webapp log') as g:
-        g.custom_command('tail', 'get_streaming_log', validator=validate_app_or_slot_exists_in_rg)
+        g.custom_command('tail', 'get_streaming_log')
         g.custom_command('download', 'download_historical_logs')
-        g.custom_command('config', 'config_diagnostics', validator=validate_app_or_slot_exists_in_rg)
-        g.custom_show_command('show', 'show_diagnostic_settings', validator=validate_app_or_slot_exists_in_rg)
+        g.custom_command('config', 'config_diagnostics')
+        g.custom_show_command('show', 'show_diagnostic_settings')
 
     with self.command_group('webapp log deployment', is_preview=True) as g:
         g.custom_show_command('show', 'show_deployment_log')
@@ -247,6 +249,10 @@ def load_command_table(self, _):
         g.custom_command('config', 'enable_cd')
         g.custom_command('show-cd-url', 'show_container_cd_url')
 
+    with self.command_group('webapp deployment github-actions', is_preview=True) as g:
+        g.custom_command('add', 'add_github_actions')
+        g.custom_command('remove', 'remove_github_actions')
+
     with self.command_group('webapp auth') as g:
         g.custom_show_command('show', 'get_auth_settings')
         g.custom_command('update', 'update_auth_settings')
@@ -283,9 +289,10 @@ def load_command_table(self, _):
                          exception_handler=ex_handler_factory(creating_plan=True))
         g.command('delete', 'delete', confirmation=True)
         g.custom_command('list', 'list_app_service_plans')
-        g.show_command('show', 'get')
-        g.generic_update_command('update', custom_func_name='update_app_service_plan', setter_arg_name='app_service_plan',
-                                 validator=validate_asp_sku, supports_no_wait=True)
+        g.custom_show_command('show', 'show_plan')
+        g.generic_update_command('update', setter_name='begin_create_or_update', custom_func_name='update_app_service_plan',
+                                 setter_arg_name='app_service_plan', supports_no_wait=True,
+                                 exception_handler=ex_handler_factory())
 
     with self.command_group('appservice') as g:
         g.custom_command('list-locations', 'list_locations', transform=transform_list_location_output)
@@ -294,9 +301,9 @@ def load_command_table(self, _):
         g.custom_command('list', 'appservice_list_vnet')
 
     with self.command_group('functionapp') as g:
-        g.custom_command('create', 'create_function', exception_handler=ex_handler_factory())
+        g.custom_command('create', 'create_functionapp', exception_handler=ex_handler_factory())
         g.custom_command('list', 'list_function_app', table_transformer=transform_web_list_output)
-        g.custom_show_command('show', 'show_webapp', table_transformer=transform_web_output)
+        g.custom_show_command('show', 'show_functionapp', table_transformer=transform_web_output)
         g.custom_command('delete', 'delete_function_app')
         g.custom_command('stop', 'stop_webapp')
         g.custom_command('start', 'start_webapp')
@@ -305,8 +312,9 @@ def load_command_table(self, _):
         g.custom_command('identity assign', 'assign_identity')
         g.custom_show_command('identity show', 'show_identity')
         g.custom_command('identity remove', 'remove_identity')
-        g.generic_update_command('update', setter_name='set_functionapp', exception_handler=update_function_ex_handler_factory(),
-                                 custom_func_name='update_functionapp', setter_type=appservice_custom, command_type=webapp_sdk)
+        g.custom_command('deploy', 'perform_onedeploy', validator=validate_onedeploy_params, is_preview=True)
+        g.generic_update_command('update', getter_name="get_functionapp", setter_name='set_functionapp', exception_handler=update_function_ex_handler_factory(),
+                                 custom_func_name='update_functionapp', getter_type=appservice_custom, setter_type=appservice_custom, command_type=webapp_sdk)
 
     with self.command_group('functionapp config') as g:
         g.custom_command('set', 'update_site_configs')
@@ -357,7 +365,8 @@ def load_command_table(self, _):
 
     with self.command_group('functionapp plan', appservice_plan_sdk) as g:
         g.custom_command('create', 'create_functionapp_app_service_plan', exception_handler=ex_handler_factory())
-        g.generic_update_command('update', custom_func_name='update_functionapp_app_service_plan',
+        g.generic_update_command('update', setter_name='begin_create_or_update',
+                                 custom_func_name='update_functionapp_app_service_plan',
                                  setter_arg_name='app_service_plan', exception_handler=ex_handler_factory())
         g.command('delete', 'delete', confirmation=True)
         g.custom_command('list', 'list_app_service_plans')
@@ -372,9 +381,6 @@ def load_command_table(self, _):
         g.custom_command('delete', 'delete_container_settings')
         g.custom_show_command('show', 'show_container_settings_functionapp')
 
-    with self.command_group('functionapp devops-pipeline') as g:
-        g.custom_command('create', 'create_devops_pipeline')
-
     with self.command_group('functionapp deployment slot') as g:
         g.custom_command('list', 'list_slots', table_transformer=output_slots_in_table)
         g.custom_command('delete', 'delete_slot')
@@ -416,35 +422,54 @@ def load_command_table(self, _):
         g.custom_command('create', 'create_appserviceenvironment_arm', supports_no_wait=True)
         g.custom_command('update', 'update_appserviceenvironment', supports_no_wait=True)
         g.custom_command('delete', 'delete_appserviceenvironment', supports_no_wait=True, confirmation=True)
+        g.custom_command('create-inbound-services', 'create_ase_inbound_services', is_preview=True)
 
     with self.command_group('appservice domain', custom_command_type=appservice_domains, is_preview=True) as g:
         g.custom_command('create', 'create_domain')
         g.custom_command('show-terms', 'show_domain_purchase_terms')
 
-    with self.command_group('staticwebapp', custom_command_type=staticsite_sdk, is_preview=True) as g:
+    with self.command_group('staticwebapp', custom_command_type=staticsite_sdk) as g:
         g.custom_command('list', 'list_staticsites')
-        g.custom_command('browse', 'show_staticsite')
+        g.custom_show_command('show', 'show_staticsite')
         g.custom_command('create', 'create_staticsites', supports_no_wait=True)
         g.custom_command('delete', 'delete_staticsite', supports_no_wait=True, confirmation=True)
         g.custom_command('disconnect', 'disconnect_staticsite', supports_no_wait=True)
         g.custom_command('reconnect', 'reconnect_staticsite', supports_no_wait=True)
+        g.custom_command('update', 'update_staticsite', supports_no_wait=True)
 
-    with self.command_group('staticwebapp environment', custom_command_type=staticsite_sdk, is_preview=True) as g:
+    with self.command_group('staticwebapp environment', custom_command_type=staticsite_sdk) as g:
         g.custom_command('list', 'list_staticsite_environments')
         g.custom_show_command('show', 'show_staticsite_environment')
         g.custom_command('functions', 'list_staticsite_functions')
+        g.custom_command('delete', 'delete_staticsite_environment', confirmation=True)
 
-    with self.command_group('staticwebapp hostname', custom_command_type=staticsite_sdk, is_preview=True) as g:
+    with self.command_group('staticwebapp hostname', custom_command_type=staticsite_sdk) as g:
         g.custom_command('list', 'list_staticsite_domains')
         g.custom_command('set', 'set_staticsite_domain', supports_no_wait=True)
         g.custom_command('delete', 'delete_staticsite_domain', supports_no_wait=True, confirmation=True)
+        g.custom_show_command('show', 'get_staticsite_domain')
 
-    with self.command_group('staticwebapp appsettings', custom_command_type=staticsite_sdk, is_preview=True) as g:
+    with self.command_group('staticwebapp appsettings', custom_command_type=staticsite_sdk) as g:
         g.custom_command('list', 'list_staticsite_function_app_settings')
         g.custom_command('set', 'set_staticsite_function_app_settings')
         g.custom_command('delete', 'delete_staticsite_function_app_settings')
 
-    with self.command_group('staticwebapp users', custom_command_type=staticsite_sdk, is_preview=True) as g:
+    with self.command_group('staticwebapp users', custom_command_type=staticsite_sdk) as g:
         g.custom_command('list', 'list_staticsite_users')
         g.custom_command('invite', 'invite_staticsite_users')
         g.custom_command('update', 'update_staticsite_users')
+
+    with self.command_group('staticwebapp secrets', custom_command_type=staticsite_sdk) as g:
+        g.custom_command('list', 'list_staticsite_secrets')
+        g.custom_command('reset-api-key', 'reset_staticsite_api_key', supports_no_wait=True)
+
+    with self.command_group('logicapp') as g:
+        g.custom_command('delete', 'delete_function_app', confirmation=True)
+        g.custom_command('stop', 'stop_webapp')
+        g.custom_command('start', 'start_webapp')
+        g.custom_command('restart', 'restart_webapp')
+
+    with self.command_group('logicapp', custom_command_type=logicapp_custom) as g:
+        g.custom_command('create', 'create_logicapp', exception_handler=ex_handler_factory())
+        g.custom_command('list', 'list_logicapp', table_transformer=transform_web_list_output)
+        g.custom_show_command('show', 'show_logicapp', table_transformer=transform_web_output)
