@@ -3,14 +3,15 @@
 # Licensed under the MIT License. See License.txt in the project root for license information.
 # --------------------------------------------------------------------------------------------
 import unittest
-import mock
+from unittest import mock
 
 from azure.cli.command_modules.appservice.static_sites import \
     list_staticsites, show_staticsite, delete_staticsite, create_staticsites, CLIError, disconnect_staticsite, \
     reconnect_staticsite, list_staticsite_environments, show_staticsite_environment, list_staticsite_domains, \
     set_staticsite_domain, delete_staticsite_domain, list_staticsite_functions, list_staticsite_function_app_settings, \
     set_staticsite_function_app_settings, delete_staticsite_function_app_settings, list_staticsite_users, \
-    invite_staticsite_users, update_staticsite_users
+    invite_staticsite_users, update_staticsite_users, update_staticsite, list_staticsite_secrets, \
+    reset_staticsite_api_key, delete_staticsite_environment
 
 
 class TestStaticAppCommands(unittest.TestCase):
@@ -70,14 +71,14 @@ class TestStaticAppCommands(unittest.TestCase):
     def test_delete_staticapp_with_resourcegroup(self):
         delete_staticsite(self.mock_cmd, self.name1, self.rg1)
 
-        self.staticapp_client.delete_static_site.assert_called_once_with(resource_group_name=self.rg1, name=self.name1)
+        self.staticapp_client.begin_delete_static_site.assert_called_once_with(resource_group_name=self.rg1, name=self.name1)
 
     def test_delete_staticapp_without_resourcegroup(self):
         self.staticapp_client.list.return_value = [self.app1, self.app2]
 
         delete_staticsite(self.mock_cmd, self.name1)
 
-        self.staticapp_client.delete_static_site.assert_called_once_with(resource_group_name=self.rg1, name=self.name1)
+        self.staticapp_client.begin_delete_static_site.assert_called_once_with(resource_group_name=self.rg1, name=self.name1)
 
     def test_delete_staticapp_not_exist(self):
         with self.assertRaises(CLIError):
@@ -88,17 +89,17 @@ class TestStaticAppCommands(unittest.TestCase):
         self.mock_cmd.get_models.return_value = StaticSiteARMResource, StaticSiteBuildProperties, SkuDescription
         app_location = './src'
         api_location = './api/'
-        app_artifact_location = '/.git/'
+        output_location = '/.git/'
         tags = {'key1': 'value1'}
 
         create_staticsites(
             self.mock_cmd, self.rg1, self.name1, self.location1,
             self.source1, self.branch1, self.token1,
-            app_location=app_location, api_location=api_location, app_artifact_location=app_artifact_location,
+            app_location=app_location, api_location=api_location, output_location=output_location,
             tags=tags)
 
-        self.staticapp_client.create_or_update_static_site.assert_called_once()
-        arg_list = self.staticapp_client.create_or_update_static_site.call_args.kwargs
+        self.staticapp_client.begin_create_or_update_static_site.assert_called_once()
+        arg_list = self.staticapp_client.begin_create_or_update_static_site.call_args[1]
         self.assertEqual(self.name1, arg_list["name"])
         self.assertEqual(self.rg1, arg_list["resource_group_name"])
         self.assertEqual(self.location1, arg_list["static_site_envelope"].location)
@@ -108,32 +109,89 @@ class TestStaticAppCommands(unittest.TestCase):
         self.assertEqual('Free', arg_list["static_site_envelope"].sku.name)
         self.assertEqual(app_location, arg_list["static_site_envelope"].build_properties.app_location)
         self.assertEqual(api_location, arg_list["static_site_envelope"].build_properties.api_location)
-        self.assertEqual(app_artifact_location, arg_list["static_site_envelope"].build_properties.app_artifact_location)
+        self.assertEqual(output_location, arg_list["static_site_envelope"].build_properties.app_artifact_location)
+
+    def test_create_staticapp_with_standard_sku(self):
+        from azure.mgmt.web.models import StaticSiteARMResource, StaticSiteBuildProperties, SkuDescription
+        self.mock_cmd.get_models.return_value = StaticSiteARMResource, StaticSiteBuildProperties, SkuDescription
+
+        create_staticsites(
+            self.mock_cmd, self.rg1, self.name1, self.location1,
+            self.source1, self.branch1, self.token1, sku='standard')
+
+        self.staticapp_client.begin_create_or_update_static_site.assert_called_once()
+        arg_list = self.staticapp_client.begin_create_or_update_static_site.call_args[1]
+        self.assertEqual('Standard', arg_list["static_site_envelope"].sku.name)
 
     def test_create_staticapp_missing_token(self):
         app_location = './src'
         api_location = './api/'
-        app_artifact_location = '/.git/'
+        output_location = '/.git/'
         tags = {'key1': 'value1'}
 
         with self.assertRaises(CLIError):
             create_staticsites(
                 self.mock_cmd, self.rg1, self.name1, self.location1,
                 self.source1, self.branch1,
-                app_location=app_location, api_location=api_location, app_artifact_location=app_artifact_location,
+                app_location=app_location, api_location=api_location, output_location=output_location,
                 tags=tags)
 
+    def test_update_staticapp(self):
+        from azure.mgmt.web.models import StaticSiteARMResource, SkuDescription
+        self.mock_cmd.get_models.return_value = StaticSiteARMResource, SkuDescription
+        self.staticapp_client.get_static_site.return_value = self.app1
+        self.staticapp_client.list.return_value = [self.app1, self.app2]
+        tags = {'key1': 'value1'}
+        sku = 'Standard'
+
+        update_staticsite(self.mock_cmd, self.name1, self.source2, self.branch2, self.token2, tags=tags, sku=sku)
+
+        self.staticapp_client.update_static_site.assert_called_once()
+        arg_list = self.staticapp_client.update_static_site.call_args[1]
+        self.assertEqual(self.name1, arg_list["name"])
+        self.assertEqual(self.source2, arg_list["static_site_envelope"].repository_url)
+        self.assertEqual(self.branch2, arg_list["static_site_envelope"].branch)
+        self.assertEqual(self.token2, arg_list["static_site_envelope"].repository_token)
+        self.assertEqual(tags, arg_list["static_site_envelope"].tags)
+        self.assertEqual(sku, arg_list["static_site_envelope"].sku.name)
+
+    def test_update_staticapp_with_no_values_passed_in(self):
+        from azure.mgmt.web.models import StaticSiteARMResource, SkuDescription
+        self.mock_cmd.get_models.return_value = StaticSiteARMResource, SkuDescription
+        self.staticapp_client.get_static_site.return_value = self.app1
+        self.staticapp_client.list.return_value = [self.app1, self.app2]
+
+        update_staticsite(self.mock_cmd, self.name1)
+
+        self.staticapp_client.update_static_site.assert_called_once()
+        arg_list = self.staticapp_client.update_static_site.call_args[1]
+        self.assertEqual(self.name1, arg_list["name"])
+        self.assertEqual(self.source1, arg_list["static_site_envelope"].repository_url)
+        self.assertEqual(self.branch1, arg_list["static_site_envelope"].branch)
+        self.assertEqual(self.token1, arg_list["static_site_envelope"].repository_token)
+        self.assertEqual(self.app1.tags, arg_list["static_site_envelope"].tags)
+        self.assertEqual('Free', arg_list["static_site_envelope"].sku.name)
+
+    def test_update_staticapp_not_exist(self):
+        from azure.mgmt.web.models import StaticSiteARMResource, SkuDescription
+        self.mock_cmd.get_models.return_value = StaticSiteARMResource, SkuDescription
+        self.staticapp_client.get_static_site.return_value = self.app1
+        self.staticapp_client.list.return_value = [self.app1, self.app2]
+
+        with self.assertRaises(CLIError):
+            update_staticsite(self.mock_cmd, self.name1_not_exist)
+
     def test_disconnect_staticapp_with_resourcegroup(self):
         disconnect_staticsite(self.mock_cmd, self.name1, self.rg1)
 
-        self.staticapp_client.detach_static_site.assert_called_once_with(resource_group_name=self.rg1, name=self.name1)
+        self.staticapp_client.begin_detach_static_site.assert_called_once_with(resource_group_name=self.rg1, name=self.name1)
 
     def test_disconnect_staticapp_without_resourcegroup(self):
         self.staticapp_client.list.return_value = [self.app1, self.app2]
 
         disconnect_staticsite(self.mock_cmd, self.name1)
 
-        self.staticapp_client.detach_static_site.assert_called_once_with(resource_group_name=self.rg1, name=self.name1)
+        self.staticapp_client.begin_detach_static_site.assert_called_once_with(resource_group_name=self.rg1, name=self.name1)
 
     @mock.patch('azure.cli.command_modules.appservice.static_sites.create_staticsites', autospec=True)
     def test_reconnect_staticapp_with_resourcegroup(self, create_staticsites_mock):
@@ -143,7 +201,7 @@ class TestStaticAppCommands(unittest.TestCase):
                              resource_group_name=self.rg1)
 
         create_staticsites_mock.assert_called_once_with(self.mock_cmd, self.rg1, self.name1, self.location1,
-                                                        self.source1, self.branch1, self.token1, no_wait=False)
+                                                        self.source1, self.branch1, self.token1, login_with_github=False, no_wait=False)
 
     @mock.patch('azure.cli.command_modules.appservice.static_sites.create_staticsites', autospec=True)
     def test_reconnect_staticapp_without_resourcegroup(self, create_staticsites_mock):
@@ -152,7 +210,7 @@ class TestStaticAppCommands(unittest.TestCase):
         reconnect_staticsite(self.mock_cmd, self.name1, self.source1, self.branch1, self.token1)
 
         create_staticsites_mock.assert_called_once_with(self.mock_cmd, self.rg1, self.name1, self.location1,
-                                                        self.source1, self.branch1, self.token1, no_wait=False)
+                                                        self.source1, self.branch1, self.token1, login_with_github=False, no_wait=False)
 
     def test_list_staticsite_environments_with_resourcegroup(self):
         list_staticsite_environments(self.mock_cmd, self.name1, self.rg1)
@@ -181,25 +239,27 @@ class TestStaticAppCommands(unittest.TestCase):
     def test_set_staticsite_domain_with_resourcegroup(self):
         set_staticsite_domain(self.mock_cmd, self.name1, self.hostname1, self.rg1)
 
-        self.staticapp_client.validate_custom_domain_can_be_added_to_static_site.assert_called_once_with(
-            self.rg1, self.name1, self.hostname1)
-        self.staticapp_client.create_or_update_static_site_custom_domain.assert_called_once_with(
-            resource_group_name=self.rg1, name=self.name1, domain_name=self.hostname1)
+        self.staticapp_client.begin_validate_custom_domain_can_be_added_to_static_site.assert_called_once_with(
+            self.rg1, self.name1, self.hostname1, self.hostname1_validation)
+        self.staticapp_client.begin_create_or_update_static_site_custom_domain.assert_called_once_with(
+            resource_group_name=self.rg1, name=self.name1, domain_name=self.hostname1,
+            static_site_custom_domain_request_properties_envelope=self.hostname1_validation)
 
     def test_set_staticsite_domain_without_resourcegroup(self):
         self.staticapp_client.list.return_value = [self.app1, self.app2]
 
         set_staticsite_domain(self.mock_cmd, self.name1, self.hostname1)
 
-        self.staticapp_client.validate_custom_domain_can_be_added_to_static_site.assert_called_once_with(
-            self.rg1, self.name1, self.hostname1)
-        self.staticapp_client.create_or_update_static_site_custom_domain.assert_called_once_with(
-            resource_group_name=self.rg1, name=self.name1, domain_name=self.hostname1)
+        self.staticapp_client.begin_validate_custom_domain_can_be_added_to_static_site.assert_called_once_with(
+            self.rg1, self.name1, self.hostname1, self.hostname1_validation)
+        self.staticapp_client.begin_create_or_update_static_site_custom_domain.assert_called_once_with(
+            resource_group_name=self.rg1, name=self.name1, domain_name=self.hostname1,
+            static_site_custom_domain_request_properties_envelope=self.hostname1_validation)
 
     def test_delete_staticsite_domain_with_resourcegroup(self):
         delete_staticsite_domain(self.mock_cmd, self.name1, self.hostname1, self.rg1)
 
-        self.staticapp_client.delete_static_site_custom_domain.assert_called_once_with(
+        self.staticapp_client.begin_delete_static_site_custom_domain.assert_called_once_with(
             resource_group_name=self.rg1, name=self.name1, domain_name=self.hostname1)
 
     def test_delete_staticsite_domain_without_resourcegroup(self):
@@ -207,9 +267,21 @@ class TestStaticAppCommands(unittest.TestCase):
 
         delete_staticsite_domain(self.mock_cmd, self.name1, self.hostname1)
 
-        self.staticapp_client.delete_static_site_custom_domain.assert_called_once_with(
+        self.staticapp_client.begin_delete_static_site_custom_domain.assert_called_once_with(
             resource_group_name=self.rg1, name=self.name1, domain_name=self.hostname1)
 
+    def test_delete_staticsite_environment_with_resourcegroup(self):
+        delete_staticsite_environment(self.mock_cmd, self.name1, self.environment1, self.rg1)
+
+        self.staticapp_client.begin_delete_static_site_build.assert_called_once_with(self.rg1, self.name1, self.environment1)
+
+    def test_delete_staticsite_environment_without_resourcegroup(self):
+        self.staticapp_client.list.return_value = [self.app1, self.app2]
+
+        delete_staticsite_environment(self.mock_cmd, self.name1, self.environment1)
+
+        self.staticapp_client.begin_delete_static_site_build.assert_called_once_with(self.rg1, self.name1, self.environment1)
+
     def test_list_staticsite_functions_with_resourcegroup(self):
         list_staticsite_functions(self.mock_cmd, self.name1, self.rg1, self.environment1)
 
@@ -239,23 +311,23 @@ class TestStaticAppCommands(unittest.TestCase):
             self.rg1, self.name1)
 
     def test_set_staticsite_function_app_settings_with_resourcegroup(self):
-        app_settings1_input = ['key1=val1', 'key2=val2']
-        app_settings1_dict = {'key1': 'val1', 'key2': 'val2'}
+        app_settings1_input = ['key1=val1', 'key2=val2==', 'key3=val3=']
+        app_settings1_dict = {'key1': 'val1', 'key2': 'val2==', 'key3': 'val3='}
 
         set_staticsite_function_app_settings(self.mock_cmd, self.name1, app_settings1_input, self.rg1)
 
         self.staticapp_client.create_or_update_static_site_function_app_settings.assert_called_once_with(
-            self.rg1, self.name1, kind=None, properties=app_settings1_dict)
+            self.rg1, self.name1, app_settings=app_settings1_dict)
 
     def test_set_staticsite_function_app_settings_without_resourcegroup(self):
-        app_settings1_input = ['key1=val1', 'key2=val2']
-        app_settings1_dict = {'key1': 'val1', 'key2': 'val2'}
+        app_settings1_input = ['key1=val1', 'key2=val2==', 'key3=val3=']
+        app_settings1_dict = {'key1': 'val1', 'key2': 'val2==', 'key3': 'val3='}
         self.staticapp_client.list.return_value = [self.app1, self.app2]
 
         set_staticsite_function_app_settings(self.mock_cmd, self.name1, app_settings1_input)
 
         self.staticapp_client.create_or_update_static_site_function_app_settings.assert_called_once_with(
-            self.rg1, self.name1, kind=None, properties=app_settings1_dict)
+            self.rg1, self.name1, app_settings=app_settings1_dict)
 
     def test_delete_staticsite_function_app_settings_with_resourcegroup(self):
         # setup
@@ -273,7 +345,7 @@ class TestStaticAppCommands(unittest.TestCase):
 
         # validate
         self.staticapp_client.create_or_update_static_site_function_app_settings.assert_called_once_with(
-            self.rg1, self.name1, kind=None, properties=updated_app_settings)
+            self.rg1, self.name1, app_settings=updated_app_settings)
 
     def test_delete_staticsite_function_app_settings_without_resourcegroup(self):
         # setup
@@ -292,7 +364,7 @@ class TestStaticAppCommands(unittest.TestCase):
 
         # validate
         self.staticapp_client.create_or_update_static_site_function_app_settings.assert_called_once_with(
-            self.rg1, self.name1, kind=None, properties=updated_app_settings)
+            self.rg1, self.name1, app_settings=updated_app_settings)
 
     def test_list_staticsite_users_with_resourcegroup(self):
         authentication_provider = 'GitHub'
@@ -322,7 +394,7 @@ class TestStaticAppCommands(unittest.TestCase):
         invite_staticsite_users(self.mock_cmd, self.name1, authentication_provider, user_details, self.hostname1,
                                 roles, invitation_expiration_in_hours, self.rg1)
 
-        arg_list = self.staticapp_client.create_user_roles_invitation_link.call_args.args
+        arg_list = self.staticapp_client.create_user_roles_invitation_link.call_args[0]
 
         self.assertEqual(self.rg1, arg_list[0])
         self.assertEqual(self.name1, arg_list[1])
@@ -343,7 +415,7 @@ class TestStaticAppCommands(unittest.TestCase):
         invite_staticsite_users(self.mock_cmd, self.name1, authentication_provider, user_details, self.hostname1,
                                 roles, invitation_expiration_in_hours)
 
-        arg_list = self.staticapp_client.create_user_roles_invitation_link.call_args.args
+        arg_list = self.staticapp_client.create_user_roles_invitation_link.call_args[0]
 
         self.assertEqual(self.rg1, arg_list[0])
         self.assertEqual(self.name1, arg_list[1])
@@ -446,6 +518,32 @@ class TestStaticAppCommands(unittest.TestCase):
             update_staticsite_users(self.mock_cmd, self.name1, roles, authentication_provider=authentication_provider,
                                     resource_group_name=self.rg1)
 
+    def test_list_staticsite_secrets(self):
+        from azure.mgmt.web.models import StringDictionary
+        self.staticapp_client.list_static_site_secrets.return_value = StringDictionary(properties={"apiKey": "key"})
+
+        secret = list_staticsite_secrets(self.mock_cmd, self.name1, self.rg1)
+
+        self.staticapp_client.list_static_site_secrets.assert_called_once_with(resource_group_name=self.rg1, name=self.name1)
+        from ast import literal_eval
+        self.assertEqual(literal_eval(secret.__str__())["properties"]["apiKey"], "key")
+
+    def test_reset_staticsite_api_key(self):
+        from azure.mgmt.web.models import StringDictionary, StaticSiteResetPropertiesARMResource
+        self.staticapp_client.get_static_site.return_value = self.app1
+        self.staticapp_client.reset_static_site_api_key.return_value = StringDictionary(properties={"apiKey": "new_key"})
+        self.mock_cmd.get_models.return_value = StaticSiteResetPropertiesARMResource
+
+        secret = reset_staticsite_api_key(self.mock_cmd, self.name1, self.rg1)
+
+        self.staticapp_client.get_static_site.assert_called_once_with(self.rg1, self.name1)
+        self.mock_cmd.get_models.assert_called_once_with('StaticSiteResetPropertiesARMResource')
+        self.staticapp_client.reset_static_site_api_key.assert_called_once()
+
+        from ast import literal_eval
+        reset_envelope = literal_eval(self.staticapp_client.reset_static_site_api_key.call_args[1]["reset_properties_envelope"].__str__())
+        self.assertEqual(reset_envelope["repository_token"], self.token1)
+
 
 def _set_up_client_mock(self):
     self.mock_cmd = mock.MagicMock()
@@ -460,6 +558,8 @@ def _set_up_client_mock(self):
 
 
 def _set_up_fake_apps(self):
+    from azure.mgmt.web.models import StaticSiteCustomDomainRequestPropertiesARMResource
+
     self.rg1 = 'rg1'
     self.name1 = 'name1'
     self.name1_not_exist = 'name1_not_exist'
@@ -469,9 +569,10 @@ def _set_up_fake_apps(self):
     self.token1 = 'TOKEN_1'
     self.environment1 = 'default'
     self.hostname1 = 'www.app1.com'
+    self.hostname1_validation = StaticSiteCustomDomainRequestPropertiesARMResource(validation_method="cname-delegation")
     self.app1 = _contruct_static_site_object(
         self.rg1, self.name1, self.location1,
-        self.source1, self.branch1)
+        self.source1, self.branch1, self.token1)
 
     self.rg2 = 'rg2'
     self.name2 = 'name2'
@@ -483,12 +584,17 @@ def _set_up_fake_apps(self):
     self.hostname1 = 'www.app2.com'
     self.app2 = _contruct_static_site_object(
         self.rg2, self.name2, self.location2,
-        self.source2, self.branch2)
+        self.source2, self.branch2, self.token2)
 
 
-def _contruct_static_site_object(rg, app_name, location, source, branch):
-    from azure.mgmt.web.models import StaticSiteARMResource
-    app = StaticSiteARMResource(location=location, repository_url=source, branch=branch)
+def _contruct_static_site_object(rg, app_name, location, source, branch, token):
+    from azure.mgmt.web.models import StaticSiteARMResource, SkuDescription
+    app = StaticSiteARMResource(
+        location=location,
+        repository_url=source,
+        branch=branch,
+        repository_token=token,
+        sku=SkuDescription(name='Free', tier='Free'))
     app.name = app_name
     app.id = \
         "/subscriptions/sub/resourceGroups/{}/providers/Microsoft.Web/staticSites/{}".format(rg, app_name)
